@@ -9,6 +9,7 @@
  * distributed with the Net-SNMP package.
  */
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 
 #include <sys/types.h>
 
@@ -21,6 +22,9 @@
 
 #include <net-snmp/agent/bulk_to_next.h>
 
+netsnmp_feature_child_of(agent_handler, libnetsnmpagent)
+
+netsnmp_feature_child_of(handler_mark_requests_as_delegated, agent_handler)
 
 static netsnmp_mib_handler *_clone_handler(netsnmp_mib_handler *it);
 
@@ -576,6 +580,7 @@ netsnmp_call_handlers(netsnmp_handler_registration *reginfo,
             return SNMP_ERR_NOERROR;    /* legal */
         break;
 
+#ifndef NETSNMP_NO_WRITE_SUPPORT
     case MODE_SET_RESERVE1:
     case MODE_SET_RESERVE2:
     case MODE_SET_ACTION:
@@ -590,6 +595,7 @@ netsnmp_call_handlers(netsnmp_handler_registration *reginfo,
             return SNMP_ERR_NOERROR;
         }
         break;
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
 
     default:
         snmp_log(LOG_ERR, "unknown mode in netsnmp_call_handlers! bug!\n");
@@ -641,6 +647,8 @@ netsnmp_call_next_handler(netsnmp_mib_handler *current,
  *
  *  @return Returns SNMPERR_SUCCESS or SNMP_ERR_* error code.
  */
+netsnmp_feature_child_of(netsnmp_call_next_handler_one_request,netsnmp_unused)
+#ifndef NETSNMP_FEATURE_REMOVE_NETSNMP_CALL_NEXT_HANDLER_ONE_REQUEST
 NETSNMP_INLINE int
 netsnmp_call_next_handler_one_request(netsnmp_mib_handler *current,
                                       netsnmp_handler_registration *reginfo,
@@ -662,6 +670,7 @@ netsnmp_call_next_handler_one_request(netsnmp_mib_handler *current,
     requests->next = request;
     return ret;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_NETSNMP_CALL_NEXT_HANDLER_ONE_REQUEST */
 
 /** Deallocates resources associated with a given handler.
  *  The handler is removed from chain and then freed.
@@ -704,27 +713,41 @@ netsnmp_handler_dup(netsnmp_mib_handler *handler)
 {
     netsnmp_mib_handler *h = NULL;
 
-    if (handler == NULL) {
-        return NULL;
-    }
+    if (!handler)
+        goto err;
 
     h = _clone_handler(handler);
+    if (!h)
+        goto err;
 
-    if (h != NULL) {
+    /*
+     * Providing a clone function without a free function is asking for
+     * memory leaks, and providing a free function without clone function
+     * is asking for memory corruption. Hence the log statement below.
+     */
+    if (!!handler->data_clone != !!handler->data_free)
+        snmp_log(LOG_ERR, "data_clone / data_free inconsistent (%s)\n",
+                 handler->handler_name);
+    if (handler->myvoid && handler->data_clone) {
+        h->myvoid = handler->data_clone(handler->myvoid);
+        if (!h->myvoid)
+            goto err;
+    } else
         h->myvoid = handler->myvoid;
-        h->data_free = handler->data_free;
+    h->data_clone = handler->data_clone;
+    h->data_free = handler->data_free;
 
-        if (handler->next != NULL) {
-            h->next = netsnmp_handler_dup(handler->next);
-            if (h->next == NULL) {
-                netsnmp_handler_free(h);
-                return NULL;
-            }
-            h->next->prev = h;
-        }
-        h->prev = NULL;
-        return h;
+    if (handler->next != NULL) {
+        h->next = netsnmp_handler_dup(handler->next);
+        if (!h->next)
+            goto err;
+        h->next->prev = h;
     }
+    h->prev = NULL;
+    return h;
+
+err:
+    netsnmp_handler_free(h);
     return NULL;
 }
 
@@ -751,7 +774,7 @@ netsnmp_handler_registration_free(netsnmp_handler_registration *reginfo)
 /** Duplicates handler registration object and all subsequent handlers.
  *  Creates a copy of the handler registration object and all its data.
  *
- *  @param handler is the handler registration object to be duplicated
+ *  @param reginfo is the handler registration object to be duplicated
  *
  *  @return Returns a pointer to the complete copy,
  *         or NULL if any problem occured.
@@ -894,11 +917,13 @@ netsnmp_free_delegated_cache(netsnmp_delegated_cache *dcache)
 }
 
 
+#ifndef NETSNMP_FEATURE_REMOVE_HANDLER_MARK_REQUESTS_AS_DELEGATED
 /** Sets a list of requests as delegated or not delegated.
  *  Sweeps through given chain of requests and sets 'delegated'
  *  flag accordingly to the isdelegaded parameter.
  *
- *  @param isdelegaded New value of the 'delegated' flag.
+ *  @param requests Request list.
+ *  @param isdelegated New value of the 'delegated' flag.
  */
 void
 netsnmp_handler_mark_requests_as_delegated(netsnmp_request_info *requests,
@@ -909,6 +934,7 @@ netsnmp_handler_mark_requests_as_delegated(netsnmp_request_info *requests,
         requests = requests->next;
     }
 }
+#endif /* NETSNMP_FEATURE_REMOVE_HANDLER_MARK_REQUESTS_AS_DELEGATED */
 
 /** Adds data from node list to the request information structure.
  *  Data in the request can be later extracted and used by submodules.
@@ -1246,6 +1272,7 @@ netsnmp_init_handler_conf(void)
     se_add_pair_to_slist("agent_mode", strdup("GET"), MODE_GET);
     se_add_pair_to_slist("agent_mode", strdup("GETNEXT"), MODE_GETNEXT);
     se_add_pair_to_slist("agent_mode", strdup("GETBULK"), MODE_GETBULK);
+#ifndef NETSNMP_NO_WRITE_SUPPORT
     se_add_pair_to_slist("agent_mode", strdup("SET_BEGIN"),
                          MODE_SET_BEGIN);
     se_add_pair_to_slist("agent_mode", strdup("SET_RESERVE1"),
@@ -1286,6 +1313,7 @@ netsnmp_init_handler_conf(void)
     se_add_pair_to_slist("babystep_mode", strdup("post_request"),
                          MODE_BSTEP_POST_REQUEST);
     se_add_pair_to_slist("babystep_mode", strdup("original"), 0xffff);
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
 
     /*
      * xxx-rks: hmmm.. will this work for modes which are or'd together?
