@@ -9,6 +9,7 @@
  * distributed with the Net-SNMP package.
  */
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
@@ -26,7 +27,11 @@
 #include <net-snmp/agent/serialize.h>
 #include <net-snmp/agent/read_only.h>
 
+netsnmp_feature_provide(instance)
+netsnmp_feature_child_of(instance, mib_helpers)
+
 typedef struct netsnmp_num_file_instance_s {
+    int   refcnt;
     char *file_name;
     FILE *filep;
     u_char type;
@@ -39,6 +44,22 @@ typedef struct netsnmp_num_file_instance_s {
  *  @{
  */
 
+static netsnmp_num_file_instance *
+netsnmp_num_file_instance_ref(netsnmp_num_file_instance *nfi)
+{
+    nfi->refcnt++;
+    return nfi;
+}
+
+static void
+netsnmp_num_file_instance_deref(netsnmp_num_file_instance *nfi)
+{
+    if (--nfi->refcnt == 0) {
+	free(nfi->file_name);
+	free(nfi);
+    }
+}
+
 /**
  * Creates an instance helper handler, calls netsnmp_create_handler, which
  * then could be registered, using netsnmp_register_handler().
@@ -111,7 +132,7 @@ netsnmp_handler_registration *
 get_reg(const char *name,
         const char *ourname,
         const oid * reg_oid, size_t reg_oid_len,
-        void *it,
+        netsnmp_num_file_instance *it,
         int modes,
         Netsnmp_Node_Handler * scalarh, Netsnmp_Node_Handler * subhandler,
         const char *contextName)
@@ -126,7 +147,9 @@ get_reg(const char *name,
                                                 reg_oid, reg_oid_len,
                                                 modes);
         myhandler = netsnmp_create_handler(ourname, scalarh);
-        myhandler->myvoid = (void *) it;
+        myhandler->myvoid = it;
+	myhandler->data_clone = (void*(*)(void*))netsnmp_num_file_instance_ref;
+	myhandler->data_free = (void(*)(void*))netsnmp_num_file_instance_deref;
         netsnmp_inject_handler(myreg, myhandler);
     } else {
         myreg =
@@ -134,7 +157,11 @@ get_reg(const char *name,
                                                 scalarh,
                                                 reg_oid, reg_oid_len,
                                                 modes);
-        myreg->handler->myvoid = (void *) it;
+        myreg->handler->myvoid = it;
+	myreg->handler->data_clone
+	    = (void *(*)(void *))netsnmp_num_file_instance_ref;
+	myreg->handler->data_free
+	    = (void (*)(void *))netsnmp_num_file_instance_deref;
     }
     if (contextName)
         myreg->contextName = strdup(contextName);
@@ -142,6 +169,8 @@ get_reg(const char *name,
 }
 
 /* Watched 'long' instances are writable on both 32-bit and 64-bit systems  */
+netsnmp_feature_child_of(read_only_ulong_instance,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_READ_ONLY_ULONG_INSTANCE
 int
 netsnmp_register_read_only_ulong_instance(const char *name,
                                           const oid * reg_oid,
@@ -149,28 +178,34 @@ netsnmp_register_read_only_ulong_instance(const char *name,
                                           Netsnmp_Node_Handler *
                                           subhandler)
 {
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
                netsnmp_create_handler_registration(
                    name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RONLY),
                netsnmp_create_watcher_info(
                    (void *)it, sizeof(u_long),
                    ASN_UNSIGNED, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_READ_ONLY_ULONG_INSTANCE */
 
+netsnmp_feature_child_of(ulong_instance,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_ULONG_INSTANCE
 int
 netsnmp_register_ulong_instance(const char *name,
                                 const oid * reg_oid, size_t reg_oid_len,
                                 u_long * it,
                                 Netsnmp_Node_Handler * subhandler)
 {
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
                netsnmp_create_handler_registration(
                    name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RWRITE),
                netsnmp_create_watcher_info(
                    (void *)it, sizeof(u_long),
                    ASN_UNSIGNED, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_ULONG_INSTANCE */
 
+netsnmp_feature_child_of(read_only_counter32_instance,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_READ_ONLY_COUNTER32_INSTANCE
 int
 netsnmp_register_read_only_counter32_instance(const char *name,
                                               const oid * reg_oid,
@@ -179,14 +214,17 @@ netsnmp_register_read_only_counter32_instance(const char *name,
                                               Netsnmp_Node_Handler *
                                               subhandler)
 {
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
                netsnmp_create_handler_registration(
                    name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RONLY),
                netsnmp_create_watcher_info(
                    (void *)it, sizeof(u_long),
                    ASN_COUNTER, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_READ_ONLY_COUNTER32_INSTANCE */
 
+netsnmp_feature_child_of(read_only_long_instance,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_READ_ONLY_LONG_INSTANCE
 int
 netsnmp_register_read_only_long_instance(const char *name,
                                          const oid * reg_oid,
@@ -194,26 +232,32 @@ netsnmp_register_read_only_long_instance(const char *name,
                                          long *it,
                                          Netsnmp_Node_Handler * subhandler)
 {
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
                netsnmp_create_handler_registration(
                    name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RONLY),
                netsnmp_create_watcher_info(
                    (void *)it, sizeof(long), ASN_INTEGER, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_READ_ONLY_LONG_INSTANCE */
 
+netsnmp_feature_child_of(long_instance,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_LONG_INSTANCE
 int
 netsnmp_register_long_instance(const char *name,
                                const oid * reg_oid, size_t reg_oid_len,
                                long *it, Netsnmp_Node_Handler * subhandler)
 {
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
                netsnmp_create_handler_registration(
                    name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RWRITE),
                netsnmp_create_watcher_info(
                    (void *)it, sizeof(long), ASN_INTEGER, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_LONG_INSTANCE */
 
 /* Watched 'int' instances are only writable on 32-bit systems  */
+netsnmp_feature_child_of(read_only_uint_instance,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_READ_ONLY_UINT_INSTANCE
 int
 netsnmp_register_read_only_uint_instance(const char *name,
                                          const oid * reg_oid,
@@ -221,42 +265,51 @@ netsnmp_register_read_only_uint_instance(const char *name,
                                          unsigned int *it,
                                          Netsnmp_Node_Handler * subhandler)
 {
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
                netsnmp_create_handler_registration(
                    name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RONLY),
                netsnmp_create_watcher_info(
                    (void *)it, sizeof(unsigned int),
                    ASN_UNSIGNED, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_READ_ONLY_UINT_INSTANCE */
 
+netsnmp_feature_child_of(uint_instance,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_UINT_INSTANCE
 int
 netsnmp_register_uint_instance(const char *name,
                                const oid * reg_oid, size_t reg_oid_len,
                                unsigned int *it, Netsnmp_Node_Handler * subhandler)
 {
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
                netsnmp_create_handler_registration(
                    name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RWRITE),
                netsnmp_create_watcher_info(
                    (void *)it, sizeof(unsigned int),
                    ASN_UNSIGNED, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_UINT_INSTANCE */
 
+netsnmp_feature_child_of(read_only_int_instance,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_READ_ONLY_INT_INSTANCE
 int
 netsnmp_register_read_only_int_instance(const char *name,
                                 const oid * reg_oid, size_t reg_oid_len,
                                 int *it, Netsnmp_Node_Handler * subhandler)
 {
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
                netsnmp_create_handler_registration(
                    name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RONLY),
                netsnmp_create_watcher_info(
                    (void *)it, sizeof(int), ASN_INTEGER, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_READ_ONLY_INT_INSTANCE */
 
   /*
    * Compatibility with earlier (inconsistently named) routine
    */
+netsnmp_feature_child_of(register_read_only_int_instance,netsnmp_unused)
+#ifndef NETSNMP_FEATURE_REMOVE_REGISTER_READ_ONLY_INT_INSTANCE
 int
 register_read_only_int_instance(const char *name,
                                 const oid * reg_oid, size_t reg_oid_len,
@@ -266,11 +319,14 @@ register_read_only_int_instance(const char *name,
                                 reg_oid, reg_oid_len,
                                 it, subhandler);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_REGISTER_READ_ONLY_INT_INSTANCE */
 
 /*
  * Context registrations
  */
 
+netsnmp_feature_child_of(register_read_only_ulong_instance_context,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_REGISTER_READ_ONLY_ULONG_INSTANCE_CONTEXT
 int
 netsnmp_register_read_only_ulong_instance_context(const char *name,
                                                   const oid * reg_oid,
@@ -285,11 +341,14 @@ netsnmp_register_read_only_ulong_instance_context(const char *name,
           name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RONLY);
     if (myreg && contextName)
       myreg->contextName = strdup(contextName);
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
         myreg, netsnmp_create_watcher_info(
             (void *)it, sizeof(u_long), ASN_UNSIGNED, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_REGISTER_READ_ONLY_ULONG_INSTANCE_CONTEXT */
 
+netsnmp_feature_child_of(register_ulong_instance_context,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_REGISTER_ULONG_INSTANCE_CONTEXT
 int
 netsnmp_register_ulong_instance_context(const char *name,
                                         const oid * reg_oid, size_t reg_oid_len,
@@ -302,11 +361,14 @@ netsnmp_register_ulong_instance_context(const char *name,
           name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RWRITE);
     if (myreg && contextName)
       myreg->contextName = strdup(contextName);
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
         myreg, netsnmp_create_watcher_info(
             (void *)it, sizeof(u_long), ASN_UNSIGNED, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_REGISTER_ULONG_INSTANCE_CONTEXT */
 
+netsnmp_feature_child_of(register_read_only_counter32_instance_context,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_REGISTER_READ_ONLY_COUNTER32_INSTANCE_CONTEXT
 int
 netsnmp_register_read_only_counter32_instance_context(const char *name,
                                                       const oid * reg_oid,
@@ -321,11 +383,14 @@ netsnmp_register_read_only_counter32_instance_context(const char *name,
           name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RONLY);
     if (myreg && contextName)
       myreg->contextName = strdup(contextName);
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
         myreg, netsnmp_create_watcher_info(
             (void *)it, sizeof(u_long), ASN_COUNTER, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_REGISTER_READ_ONLY_COUNTER32_INSTANCE_CONTEXT */
 
+netsnmp_feature_child_of(register_read_only_long_instance_context,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_REGISTER_READ_ONLY_LONG_INSTANCE_CONTEXT
 int
 netsnmp_register_read_only_long_instance_context(const char *name,
                                                  const oid * reg_oid,
@@ -340,11 +405,14 @@ netsnmp_register_read_only_long_instance_context(const char *name,
           name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RONLY);
     if (myreg && contextName)
       myreg->contextName = strdup(contextName);
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
         myreg, netsnmp_create_watcher_info(
             (void *)it, sizeof(long), ASN_INTEGER, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_REGISTER_READ_ONLY_LONG_INSTANCE_CONTEXT */
 
+netsnmp_feature_child_of(register_long_instance_context,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_REGISTER_LONG_INSTANCE_CONTEXT
 int
 netsnmp_register_long_instance_context(const char *name,
                                        const oid * reg_oid, size_t reg_oid_len,
@@ -357,11 +425,14 @@ netsnmp_register_long_instance_context(const char *name,
           name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RWRITE);
     if (myreg && contextName)
       myreg->contextName = strdup(contextName);
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
         myreg, netsnmp_create_watcher_info(
             (void *)it, sizeof(long), ASN_INTEGER, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_REGISTER_LONG_INSTANCE_CONTEXT */
 
+netsnmp_feature_child_of(register_int_instance_context,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_REGISTER_INT_INSTANCE_CONTEXT
 int
 netsnmp_register_int_instance_context(const char *name,
                                       const oid * reg_oid,
@@ -375,11 +446,14 @@ netsnmp_register_int_instance_context(const char *name,
           name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RWRITE);
     if (myreg && contextName)
       myreg->contextName = strdup(contextName);
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
         myreg, netsnmp_create_watcher_info(
             (void *)it, sizeof(int), ASN_INTEGER, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_REGISTER_INT_INSTANCE_CONTEXT */
 
+netsnmp_feature_child_of(register_read_only_int_instance_context,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_REGISTER_READ_ONLY_INT_INSTANCE_CONTEXT
 int
 netsnmp_register_read_only_int_instance_context(const char *name,
                                                 const oid * reg_oid,
@@ -393,14 +467,17 @@ netsnmp_register_read_only_int_instance_context(const char *name,
           name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RONLY);
     if (myreg && contextName)
       myreg->contextName = strdup(contextName);
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
         myreg, netsnmp_create_watcher_info(
             (void *)it, sizeof(int), ASN_INTEGER, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_REGISTER_READ_ONLY_INT_INSTANCE_CONTEXT */
 
 /*
  * Compatibility with earlier (inconsistently named) routine
  */
+netsnmp_feature_child_of(read_only_int_instance_context,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_READ_ONLY_INT_INSTANCE_CONTEXT
 int
 register_read_only_int_instance_context(const char *name,
                                         const oid * reg_oid, size_t reg_oid_len,
@@ -413,7 +490,10 @@ register_read_only_int_instance_context(const char *name,
                                                            it, subhandler,
                                                            contextName);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_READ_ONLY_INT_INSTANCE_CONTEXT */
 
+netsnmp_feature_child_of(register_num_file_instance,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_REGISTER_NUM_FILE_INSTANCE
 int
 netsnmp_register_num_file_instance(const char *name,
                                    const oid * reg_oid, size_t reg_oid_len,
@@ -438,11 +518,12 @@ netsnmp_register_num_file_instance(const char *name,
         return MIB_REGISTRATION_FAILED;
     }
 
+    nfi->refcnt = 1;
     myreg = get_reg(name, "file_num_handler", reg_oid, reg_oid_len, nfi,
                     mode, netsnmp_instance_num_file_handler,
                     subhandler, contextName);
     if (NULL == myreg) {
-        free(nfi); /* SNMP_FREE overkill on local var */
+        netsnmp_num_file_instance_deref(nfi);
         return MIB_REGISTRATION_FAILED;
     }
 
@@ -453,7 +534,10 @@ netsnmp_register_num_file_instance(const char *name,
 
     return netsnmp_register_instance(myreg);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_REGISTER_NUM_FILE_INSTANCE */
 
+netsnmp_feature_child_of(register_int_instance,instance)
+#ifndef NETSNMP_FEATURE_REMOVE_REGISTER_INT_INSTANCE
 /**
  * This function registers an int helper handler to a specified OID.
  *
@@ -477,12 +561,13 @@ netsnmp_register_int_instance(const char *name,
                               const oid * reg_oid, size_t reg_oid_len,
                               int *it, Netsnmp_Node_Handler * subhandler)
 {
-    return netsnmp_register_watched_instance(
+    return netsnmp_register_watched_instance2(
                netsnmp_create_handler_registration(
                    name, subhandler, reg_oid, reg_oid_len, HANDLER_CAN_RWRITE),
                netsnmp_create_watcher_info(
                    (void *)it, sizeof(int), ASN_INTEGER, WATCHER_FIXED_SIZE));
 }
+#endif /* NETSNMP_FEATURE_REMOVE_REGISTER_INT_INSTANCE */
 
 #ifdef HAVE_DMALLOC_H
 static void free_wrapper(void * p)
@@ -493,290 +578,6 @@ static void free_wrapper(void * p)
 #define free_wrapper free
 #endif
 
-#ifndef NETSNMP_NO_DEPRECATED_FUNCTIONS
-
-/**
- * \deprecated This function is unused and scheduled for removal in Net-SNMP 5.6
- */
-int
-netsnmp_instance_ulong_handler(netsnmp_mib_handler *handler,
-                               netsnmp_handler_registration *reginfo,
-                               netsnmp_agent_request_info *reqinfo,
-                               netsnmp_request_info *requests)
-{
-
-    u_long         *it = (u_long *) handler->myvoid;
-    u_long         *it_save;
-
-    DEBUGMSGTL(("netsnmp_instance_ulong_handler", "Got request:  %d\n",
-                reqinfo->mode));
-
-    switch (reqinfo->mode) {
-        /*
-         * data requests 
-         */
-    case MODE_GET:
-        snmp_set_var_typed_value(requests->requestvb, ASN_UNSIGNED,
-                                 (u_char *) it, sizeof(*it));
-        break;
-
-        /*
-         * SET requests.  Should only get here if registered RWRITE 
-         */
-    case MODE_SET_RESERVE1:
-        if (requests->requestvb->type != ASN_UNSIGNED)
-            netsnmp_set_request_error(reqinfo, requests,
-                                      SNMP_ERR_WRONGTYPE);
-        break;
-
-    case MODE_SET_RESERVE2:
-        /*
-         * store old info for undo later 
-         */
-        memdup((u_char **) & it_save, (u_char *) it, sizeof(u_long));
-        if (it_save == NULL) {
-            netsnmp_set_request_error(reqinfo, requests,
-                                      SNMP_ERR_RESOURCEUNAVAILABLE);
-            return SNMP_ERR_NOERROR;
-        }
-        netsnmp_request_add_list_data(requests,
-                                      netsnmp_create_data_list
-                                      (INSTANCE_HANDLER_NAME, it_save,
-                                       &free_wrapper));
-        break;
-
-    case MODE_SET_ACTION:
-        /*
-         * update current 
-         */
-        DEBUGMSGTL(("testhandler", "updated u_long %lu -> %lu\n", *it,
-                    *(requests->requestvb->val.integer)));
-        *it = *(requests->requestvb->val.integer);
-        break;
-
-    case MODE_SET_UNDO:
-        *it =
-            *((u_long *) netsnmp_request_get_list_data(requests,
-                                                       INSTANCE_HANDLER_NAME));
-        break;
-
-    case MODE_SET_COMMIT:
-    case MODE_SET_FREE:
-        /*
-         * nothing to do 
-         */
-        break;
-    }
-
-    if (handler->next && handler->next->access_method)
-        return netsnmp_call_next_handler(handler, reginfo, reqinfo,
-                                         requests);
-
-    return SNMP_ERR_NOERROR;
-}
-
-/**
- * \deprecated This function is unused and scheduled for removal in Net-SNMP 5.6
- */
-int
-netsnmp_instance_counter32_handler(netsnmp_mib_handler *handler,
-                                   netsnmp_handler_registration *reginfo,
-                                   netsnmp_agent_request_info *reqinfo,
-                                   netsnmp_request_info *requests)
-{
-
-    u_long         *it = (u_long *) handler->myvoid;
-
-    DEBUGMSGTL(("netsnmp_instance_counter32_handler",
-                "Got request:  %d\n", reqinfo->mode));
-
-    switch (reqinfo->mode) {
-        /*
-         * data requests 
-         */
-    case MODE_GET:
-        snmp_set_var_typed_value(requests->requestvb, ASN_COUNTER,
-                                 (u_char *) it, sizeof(*it));
-        break;
-
-        /*
-         * SET requests.  Should only get here if registered RWRITE 
-         */
-    default:
-        snmp_log(LOG_ERR,
-                 "netsnmp_instance_counter32_handler: illegal mode\n");
-        netsnmp_set_request_error(reqinfo, requests, SNMP_ERR_GENERR);
-        return SNMP_ERR_NOERROR;
-    }
-    if (handler->next && handler->next->access_method)
-        return netsnmp_call_next_handler(handler, reginfo, reqinfo,
-                                         requests);
-    return SNMP_ERR_NOERROR;
-}
-
-/**
- * \deprecated This function is unused and scheduled for removal in Net-SNMP 5.6
- */
-int
-netsnmp_instance_long_handler(netsnmp_mib_handler *handler,
-                              netsnmp_handler_registration *reginfo,
-                              netsnmp_agent_request_info *reqinfo,
-                              netsnmp_request_info *requests)
-{
-
-    long           *it = (long *) handler->myvoid;
-    long           *it_save;
-
-    DEBUGMSGTL(("netsnmp_instance_long_handler", "Got request:  %d\n",
-                reqinfo->mode));
-
-    switch (reqinfo->mode) {
-        /*
-         * data requests 
-         */
-    case MODE_GET:
-        snmp_set_var_typed_value(requests->requestvb, ASN_INTEGER,
-                                 (u_char *) it, sizeof(*it));
-        break;
-
-        /*
-         * SET requests.  Should only get here if registered RWRITE 
-         */
-    case MODE_SET_RESERVE1:
-        if (requests->requestvb->type != ASN_INTEGER)
-            netsnmp_set_request_error(reqinfo, requests,
-                                      SNMP_ERR_WRONGTYPE);
-        break;
-
-    case MODE_SET_RESERVE2:
-        /*
-         * store old info for undo later 
-         */
-        memdup((u_char **) & it_save, (u_char *) it, sizeof(long));
-        if (it_save == NULL) {
-            netsnmp_set_request_error(reqinfo, requests,
-                                      SNMP_ERR_RESOURCEUNAVAILABLE);
-            return SNMP_ERR_NOERROR;
-        }
-        netsnmp_request_add_list_data(requests,
-                                      netsnmp_create_data_list
-                                      (INSTANCE_HANDLER_NAME, it_save,
-                                       &free_wrapper));
-        break;
-
-    case MODE_SET_ACTION:
-        /*
-         * update current 
-         */
-        DEBUGMSGTL(("testhandler", "updated u_long %lu -> %lu\n", *it,
-                    *(requests->requestvb->val.integer)));
-        *it = *(requests->requestvb->val.integer);
-        break;
-
-    case MODE_SET_UNDO:
-        *it =
-            *((u_long *) netsnmp_request_get_list_data(requests,
-                                                       INSTANCE_HANDLER_NAME));
-        break;
-
-    case MODE_SET_COMMIT:
-    case MODE_SET_FREE:
-        /*
-         * nothing to do 
-         */
-        break;
-    }
-    if (handler->next && handler->next->access_method)
-        return netsnmp_call_next_handler(handler, reginfo, reqinfo,
-                                         requests);
-    return SNMP_ERR_NOERROR;
-}
-
-/**
- * \deprecated This function is unused and scheduled for removal in Net-SNMP 5.6
- */
-int
-netsnmp_instance_int_handler(netsnmp_mib_handler *handler,
-                             netsnmp_handler_registration *reginfo,
-                             netsnmp_agent_request_info *reqinfo,
-                             netsnmp_request_info *requests)
-{
-
-    int *it = (int *) handler->myvoid;
-    int *it_save;
-    long tmp_it;
-    
-    DEBUGMSGTL(("netsnmp_instance_int_handler", "Got request:  %d\n",
-                reqinfo->mode));
-
-    switch (reqinfo->mode) {
-        /*
-         * data requests 
-         */
-    case MODE_GET:
-	/*
-	 * Use a long here, otherwise on 64 bit use of an int would fail
-	 */
-	tmp_it = *it;
-        snmp_set_var_typed_value(requests->requestvb, ASN_INTEGER,
-                                 (u_char *) &tmp_it, sizeof(tmp_it));
-        break;
-
-        /*
-         * SET requests.  Should only get here if registered RWRITE 
-         */
-    case MODE_SET_RESERVE1:
-        if (requests->requestvb->type != ASN_INTEGER)
-            netsnmp_set_request_error(reqinfo, requests,
-                                      SNMP_ERR_WRONGTYPE);
-        break;
-
-    case MODE_SET_RESERVE2:
-        /*
-         * store old info for undo later 
-         */
-        memdup((u_char **) & it_save, (u_char *) it, sizeof(int));
-        if (it_save == NULL) {
-            netsnmp_set_request_error(reqinfo, requests,
-                                      SNMP_ERR_RESOURCEUNAVAILABLE);
-            return SNMP_ERR_NOERROR;
-        }
-        netsnmp_request_add_list_data(requests,
-                                      netsnmp_create_data_list
-                                      (INSTANCE_HANDLER_NAME, it_save,
-                                       &free_wrapper));
-        break;
-
-    case MODE_SET_ACTION:
-        /*
-         * update current 
-         */
-        DEBUGMSGTL(("testhandler", "updated int %d -> %ld\n", *it,
-                    *(requests->requestvb->val.integer)));
-        *it = (int) *(requests->requestvb->val.integer);
-        break;
-
-    case MODE_SET_UNDO:
-        *it =
-            *((u_int *) netsnmp_request_get_list_data(requests,
-                                                      INSTANCE_HANDLER_NAME));
-        break;
-
-    case MODE_SET_COMMIT:
-    case MODE_SET_FREE:
-        /*
-         * nothing to do 
-         */
-        break;
-    }
-    if (handler->next && handler->next->access_method)
-        return netsnmp_call_next_handler(handler, reginfo, reqinfo,
-                                         requests);
-    return SNMP_ERR_NOERROR;
-}
-
-#endif /* NETSNMP_NO_DEPRECATED_FUNCTIONS */
-
 int
 netsnmp_instance_num_file_handler(netsnmp_mib_handler *handler,
                                   netsnmp_handler_registration *reginfo,
@@ -784,7 +585,10 @@ netsnmp_instance_num_file_handler(netsnmp_mib_handler *handler,
                                   netsnmp_request_info *requests)
 {
     netsnmp_num_file_instance *nfi;
-    u_long it, *it_save;
+    u_long it;
+#ifndef NETSNMP_NO_WRITE_SUPPORT
+    u_long *it_save;
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
     int rc;
 
     netsnmp_assert(NULL != handler);
@@ -826,6 +630,7 @@ netsnmp_instance_num_file_handler(netsnmp_mib_handler *handler,
         /*
          * SET requests.  Should only get here if registered RWRITE 
          */
+#ifndef NETSNMP_NO_WRITE_SUPPORT
     case MODE_SET_RESERVE1:
         netsnmp_assert(NULL == nfi->filep);
         if (requests->requestvb->type != nfi->type)
@@ -898,6 +703,12 @@ netsnmp_instance_num_file_handler(netsnmp_mib_handler *handler,
             nfi->filep = NULL;
         }
         break;
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
+    default:
+        snmp_log(LOG_ERR,
+                 "netsnmp_instance_num_file_handler: illegal mode\n");
+        netsnmp_set_request_error(reqinfo, requests, SNMP_ERR_GENERR);
+        return SNMP_ERR_NOERROR;
     }
 
     if (handler->next && handler->next->access_method)
@@ -906,93 +717,6 @@ netsnmp_instance_num_file_handler(netsnmp_mib_handler *handler,
     return SNMP_ERR_NOERROR;
 }
 
-#ifndef NETSNMP_NO_DEPRECATED_FUNCTIONS
-
-/**
- * \deprecated This function is unused and scheduled for removal in Net-SNMP 5.6
- */
-int
-netsnmp_instance_uint_handler(netsnmp_mib_handler *handler,
-                              netsnmp_handler_registration *reginfo,
-                              netsnmp_agent_request_info *reqinfo,
-                              netsnmp_request_info *requests)
-{
-
-    unsigned int *it = (unsigned int *) handler->myvoid;
-    unsigned int *it_save;
-    unsigned long tmp_it;
-    
-    DEBUGMSGTL(("netsnmp_instance_uint_handler", "Got request:  %d\n",
-                reqinfo->mode));
-
-    switch (reqinfo->mode) {
-        /*
-         * data requests 
-         */
-    case MODE_GET:
-	/*
-	 * Use a long here, otherwise on 64 bit use of an int would fail
-	 */
-	tmp_it = *it;
-        snmp_set_var_typed_value(requests->requestvb, ASN_UNSIGNED,
-                                 (u_char *) &tmp_it, sizeof(unsigned long));
-        break;
-
-        /*
-         * SET requests.  Should only get here if registered RWRITE 
-         */
-    case MODE_SET_RESERVE1:
-        if (requests->requestvb->type != ASN_UNSIGNED)
-            netsnmp_set_request_error(reqinfo, requests,
-                                      SNMP_ERR_WRONGTYPE);
-        break;
-
-    case MODE_SET_RESERVE2:
-        /*
-         * store old info for undo later 
-         */
-        memdup((u_char **) & it_save, (u_char *) it, sizeof(u_int));
-        if (it_save == NULL) {
-            netsnmp_set_request_error(reqinfo, requests,
-                                      SNMP_ERR_RESOURCEUNAVAILABLE);
-            return SNMP_ERR_NOERROR;
-        }
-        netsnmp_request_add_list_data(requests,
-                                      netsnmp_create_data_list
-                                      (INSTANCE_HANDLER_NAME, it_save,
-                                       &free_wrapper));
-        break;
-
-    case MODE_SET_ACTION:
-        /*
-         * update current 
-         */
-        DEBUGMSGTL(("testhandler", "updated uint %d -> %ld\n", *it,
-                    *(requests->requestvb->val.integer)));
-        *it = (unsigned int) *(requests->requestvb->val.integer);
-        break;
-
-    case MODE_SET_UNDO:
-        *it =
-            *((u_int *) netsnmp_request_get_list_data(requests,
-                                                      INSTANCE_HANDLER_NAME));
-        break;
-
-    case MODE_SET_COMMIT:
-    case MODE_SET_FREE:
-        /*
-         * nothing to do 
-         */
-        break;
-    }
-    if (handler->next && handler->next->access_method)
-        return netsnmp_call_next_handler(handler, reginfo, reqinfo,
-                                         requests);
-    return SNMP_ERR_NOERROR;
-}
-
-#endif /* NETSNMP_NO_DEPRECATED_FUNCTIONS */
-
 int
 netsnmp_instance_helper_handler(netsnmp_mib_handler *handler,
                                 netsnmp_handler_registration *reginfo,
@@ -1025,6 +749,7 @@ netsnmp_instance_helper_handler(netsnmp_mib_handler *handler,
         }
         break;
 
+#ifndef NETSNMP_NO_WRITE_SUPPORT
     case MODE_SET_RESERVE1:
     case MODE_SET_RESERVE2:
     case MODE_SET_ACTION:
@@ -1040,6 +765,7 @@ netsnmp_instance_helper_handler(netsnmp_mib_handler *handler,
                                              requests);
         }
         break;
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
 
     case MODE_GETNEXT:
         if (cmp < 0 || (cmp == 0 && requests->inclusive)) {
@@ -1065,6 +791,11 @@ netsnmp_instance_helper_handler(netsnmp_mib_handler *handler,
             return SNMP_ERR_NOERROR;
         }
         break;
+    default:
+        snmp_log(LOG_ERR,
+                 "netsnmp_instance_helper_handler: illegal mode\n");
+        netsnmp_set_request_error(reqinfo, requests, SNMP_ERR_GENERR);
+        return SNMP_ERR_NOERROR;
     }
     /*
      * got here only if illegal mode found 
