@@ -1,9 +1,10 @@
 /*
  * table_container.c
- * $Id: table_container.c 19204 2010-07-12 10:08:54Z bvassche $
+ * $Id$
  */
 
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
@@ -20,6 +21,17 @@
 #include <net-snmp/library/container.h>
 #include <net-snmp/library/snmp_assert.h>
 
+netsnmp_feature_provide(table_container)
+netsnmp_feature_child_of(table_container, table_container_all)
+netsnmp_feature_child_of(table_container_replace_row, table_container_all)
+netsnmp_feature_child_of(table_container_extract, table_container_all)
+netsnmp_feature_child_of(table_container_management, table_container_all)
+netsnmp_feature_child_of(table_container_row_remove, table_container_all)
+netsnmp_feature_child_of(table_container_row_insert, table_container_all)
+netsnmp_feature_child_of(table_container_all, mib_helpers)
+
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_CONTAINER
+
 /*
  * snmp.h:#define SNMP_MSG_INTERNAL_SET_BEGIN        -1 
  * snmp.h:#define SNMP_MSG_INTERNAL_SET_RESERVE1     0 
@@ -35,6 +47,9 @@
  */
 typedef struct container_table_data_s {
 
+    /** Number of handlers whose myvoid pointer points to this structure. */
+    int refcnt;
+
    /** registration info for the table */
     netsnmp_table_registration_info *tblreg_info;
 
@@ -160,6 +175,7 @@ _find_next_row(netsnmp_container *c,
  *
  * ================================== */
 
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_CONTAINER_MANAGEMENT
 container_table_data *
 netsnmp_tcontainer_create_table( const char *name,
                                  netsnmp_container *container, long flags )
@@ -204,6 +220,7 @@ netsnmp_tcontainer_delete_table( container_table_data *table )
     SNMP_FREE(table);
     return;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_CONTAINER_MANAGEMENT */
 
     /*
      * The various standalone row operation routines
@@ -230,6 +247,7 @@ netsnmp_tcontainer_remove_row( container_table_data *table, netsnmp_index *row )
     return NULL;
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_CONTAINER_REPLACE_ROW
 int
 netsnmp_tcontainer_replace_row( container_table_data *table,
                                 netsnmp_index *old_row, netsnmp_index *new_row )
@@ -240,6 +258,7 @@ netsnmp_tcontainer_replace_row( container_table_data *table,
     netsnmp_tcontainer_add_row(    table, new_row );
     return 0;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_CONTAINER_REPLACE_ROW */
 
     /* netsnmp_tcontainer_remove_delete_row() will be table-specific too */
 
@@ -250,6 +269,20 @@ netsnmp_tcontainer_replace_row( container_table_data *table,
  *
  * ================================== */
 
+static container_table_data *
+netsnmp_container_table_data_clone(container_table_data *tad)
+{
+    ++tad->refcnt;
+    return tad;
+}
+
+static void
+netsnmp_container_table_data_free(container_table_data *tad)
+{
+    if (--tad->refcnt == 0)
+	free(tad);
+}
+
 /** returns a netsnmp_mib_handler object for the table_container helper */
 netsnmp_mib_handler *
 netsnmp_container_table_handler_get(netsnmp_table_registration_info *tabreg,
@@ -274,6 +307,7 @@ netsnmp_container_table_handler_get(netsnmp_table_registration_info *tabreg,
         return NULL;
     }
 
+    tad->refcnt = 1;
     tad->tblreg_info = tabreg;  /* we need it too, but it really is not ours */
     if(key_type)
         tad->key_type = key_type;
@@ -290,6 +324,8 @@ netsnmp_container_table_handler_get(netsnmp_table_registration_info *tabreg,
         container->ncompare = netsnmp_ncompare_netsnmp_index;
     
     handler->myvoid = (void*)tad;
+    handler->data_clone = (void *(*)(void *))netsnmp_container_table_data_clone;
+    handler->data_free = (void (*)(void *))netsnmp_container_table_data_free;
     handler->flags |= MIB_HANDLER_AUTO_NEXT;
     
     return handler;
@@ -328,19 +364,23 @@ netsnmp_container_table_unregister(netsnmp_handler_registration *reginfo)
     if (tad) {
         CONTAINER_FREE( tad->table );
         tad->table = NULL;
-        free(tad);
-        reginfo->handler->myvoid = NULL;
+	/*
+	 * Note: don't free the memory tad points at here - that is done
+	 * by netsnmp_container_table_data_free().
+	 */
     }
     return netsnmp_unregister_table( reginfo );
 }
 
 /** retrieve the container used by the table_container helper */
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_CONTAINER_EXTRACT
 netsnmp_container*
 netsnmp_container_table_container_extract(netsnmp_request_info *request)
 {
     return (netsnmp_container *)
          netsnmp_request_get_list_data(request, TABLE_CONTAINER_CONTAINER);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_CONTAINER_EXTRACT */
 
 #ifndef NETSNMP_USE_INLINE
 /** find the context data used by the table_container helper */
@@ -365,6 +405,7 @@ netsnmp_container_table_extract_context(netsnmp_request_info *request)
 }
 #endif /* inline */
 
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_CONTAINER_ROW_INSERT
 /** inserts a newly created table_container entry into a request list */
 void
 netsnmp_container_table_row_insert(netsnmp_request_info *request,
@@ -432,7 +473,9 @@ netsnmp_container_table_row_insert(netsnmp_request_info *request,
         }
     }
 }
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_CONTAINER_ROW_INSERT */
 
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_CONTAINER_ROW_REMOVE
 /** removes a table_container entry from a request list */
 void
 netsnmp_container_table_row_remove(netsnmp_request_info *request,
@@ -499,6 +542,7 @@ netsnmp_container_table_row_remove(netsnmp_request_info *request,
         }
     }
 }
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_CONTAINER_ROW_REMOVE */
 
 /** @cond */
 /**********************************************************************
@@ -610,11 +654,15 @@ _data_lookup(netsnmp_handler_registration *reginfo,
              * not results found. For a get, that is an error
              */
             DEBUGMSGTL(("table_container", "no row found\n"));
+#ifndef NETSNMP_NO_WRITE_SUPPORT
             if((agtreq_info->mode != MODE_SET_RESERVE1) || /* get */
                (reginfo->modes & HANDLER_CAN_NOT_CREATE)) { /* no create */
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
                 netsnmp_set_request_error(agtreq_info, request,
                                           SNMP_NOSUCHINSTANCE);
+#ifndef NETSNMP_NO_WRITE_SUPPORT
             }
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
         }
     } /** GET/SET */
     
@@ -676,7 +724,11 @@ _container_table_handler(netsnmp_mib_handler *handler,
      * registration.
      */
     oldmode = agtreq_info->mode;
-    if(MODE_IS_GET(oldmode) || (MODE_SET_RESERVE1 == oldmode)) {
+    if(MODE_IS_GET(oldmode)
+#ifndef NETSNMP_NO_WRITE_SUPPORT
+       || (MODE_SET_RESERVE1 == oldmode)
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
+        ) {
         netsnmp_request_info *curr_request;
         /*
          * Loop through each of the requests, and
@@ -816,4 +868,9 @@ netsnmp_table_index_find_next_row(netsnmp_container *c,
  *
  * ================================== */
 
+#else /* NETSNMP_FEATURE_REMOVE_TABLE_CONTAINER */
+netsnmp_feature_unused(table_container);
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_CONTAINER */
 /** @} */
+
+
