@@ -1,4 +1,5 @@
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
@@ -14,6 +15,27 @@
 #include <net-snmp/agent/table.h>
 #include <net-snmp/agent/read_only.h>
 
+netsnmp_feature_child_of(table_data_all, mib_helpers)
+
+netsnmp_feature_child_of(table_data, table_data_all)
+netsnmp_feature_child_of(register_read_only_table_data, table_data_all)
+netsnmp_feature_child_of(extract_table_row_data, table_data_all)
+netsnmp_feature_child_of(insert_table_row, table_data_all)
+netsnmp_feature_child_of(table_data_delete_table, table_data_all)
+
+netsnmp_feature_child_of(table_data_extras, table_data_all)
+
+netsnmp_feature_child_of(table_data_create_table, table_data_extras)
+netsnmp_feature_child_of(table_data_create_row, table_data_extras)
+netsnmp_feature_child_of(table_data_copy_row, table_data_extras)
+netsnmp_feature_child_of(table_data_remove_delete_row, table_data_extras)
+netsnmp_feature_child_of(table_data_unregister, table_data_extras)
+netsnmp_feature_child_of(table_data_row_count, table_data_extras)
+netsnmp_feature_child_of(table_data_row_operations, table_data_extras)
+netsnmp_feature_child_of(table_data_row_first, table_data_extras)
+
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_DATA
+
 /** @defgroup table_data table_data
  *  Helps you implement a table with datamatted storage.
  *  @ingroup table
@@ -91,6 +113,7 @@ netsnmp_table_data_clone_row(netsnmp_table_row *row)
         newrow->index_oid =
             snmp_duplicate_objid(row->index_oid, row->index_oid_len);
         if (!newrow->index_oid) {
+            free(newrow->indexes);
             free(newrow);
             return NULL;
         }
@@ -288,12 +311,15 @@ netsnmp_table_data_remove_and_delete_row(netsnmp_table_data *table,
      * Generic API - mostly renamed wrappers
      * ===================================== */
 
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_DATA_CREATE_TABLE
 netsnmp_table_data *
 netsnmp_table_data_create_table(const char *name, long flags)
 {
     return netsnmp_create_table_data( name );
 }
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_DATA_CREATE_TABLE */
 
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_DATA_DELETE_TABLE
 void
 netsnmp_table_data_delete_table( netsnmp_table_data *table )
 {
@@ -313,14 +339,13 @@ netsnmp_table_data_delete_table( netsnmp_table_data *table )
     }
     table->first_row = NULL;
 
-    if (table->name) {
-        SNMP_FREE(table->name);
-        table->name = NULL;
-    }
+    SNMP_FREE(table->name);
     SNMP_FREE(table);
     return;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_DATA_DELETE_TABLE */
 
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_DATA_CREATE_ROW
 netsnmp_table_row *
 netsnmp_table_data_create_row( void* entry )
 {
@@ -329,9 +354,11 @@ netsnmp_table_data_create_row( void* entry )
         row->data = entry;
     return row;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_DATA_CREATE_ROW */
 
     /* netsnmp_table_data_clone_row() defined above */
 
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_DATA_COPY_ROW
 int
 netsnmp_table_data_copy_row( netsnmp_table_row  *old_row,
                              netsnmp_table_row  *new_row )
@@ -349,6 +376,7 @@ netsnmp_table_data_copy_row( netsnmp_table_row  *old_row,
     /* XXX - Doesn't copy table-specific row structure */
     return 0;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_DATA_COPY_ROW */
 
     /*
      * netsnmp_table_data_delete_row()
@@ -358,12 +386,14 @@ netsnmp_table_data_copy_row( netsnmp_table_row  *old_row,
      *     all defined above
      */
 
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_DATA_REMOVE_DELETE_ROW
 void *
 netsnmp_table_data_remove_delete_row(netsnmp_table_data *table,
                                      netsnmp_table_row *row)
 {
     return netsnmp_table_data_remove_and_delete_row(table, row);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_DATA_REMOVE_DELETE_ROW */
 
 
 /* ==================================
@@ -405,6 +435,8 @@ netsnmp_register_table_data(netsnmp_handler_registration *reginfo,
     return netsnmp_register_table(reginfo, table_info);
 }
 
+
+#ifndef NETSNMP_FEATURE_REMOVE_REGISTER_READ_ONLY_TABLE_DATA
 /** registers a handler as a read-only data table
  *  If table_info != NULL, it registers it as a normal table too. */
 int
@@ -415,13 +447,16 @@ netsnmp_register_read_only_table_data(netsnmp_handler_registration *reginfo,
     netsnmp_inject_handler(reginfo, netsnmp_get_read_only_handler());
     return netsnmp_register_table_data(reginfo, table, table_info);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_REGISTER_READ_ONLY_TABLE_DATA */
 
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_DATA_UNREGISTER
 int
 netsnmp_unregister_table_data(netsnmp_handler_registration *reginfo)
 {
     /* free table; */
     return netsnmp_unregister_table(reginfo);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_DATA_UNREGISTER */
 
 /*
  * The helper handler that takes care of passing a specific row of
@@ -454,7 +489,9 @@ netsnmp_table_data_helper_handler(netsnmp_mib_handler *handler,
         switch (reqinfo->mode) {
         case MODE_GET:
         case MODE_GETNEXT:
+#ifndef NETSNMP_NO_WRITE_SUPPORT
         case MODE_SET_RESERVE1:
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
             netsnmp_request_add_list_data(request,
                                       netsnmp_create_data_list(
                                           TABLE_DATA_TABLE, table, NULL));
@@ -528,9 +565,7 @@ netsnmp_table_data_helper_handler(netsnmp_mib_handler *handler,
                         /*
                          * equal match, return the next row 
                          */
-                        if (row) {
-                            row = row->next;
-                        }
+                        row = row->next;
                         break;
                     } else if (result > 0) {
                         /*
@@ -607,6 +642,7 @@ netsnmp_table_data_helper_handler(netsnmp_mib_handler *handler,
             }
             break;
 
+#ifndef NETSNMP_NO_WRITE_SUPPORT
         case MODE_SET_RESERVE1:
             valid_request = 1;
             if (NULL !=
@@ -631,6 +667,7 @@ netsnmp_table_data_helper_handler(netsnmp_mib_handler *handler,
         case MODE_SET_FREE:
         case MODE_SET_UNDO:
             valid_request = 1;
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
 
         }
     }
@@ -673,6 +710,7 @@ netsnmp_extract_table_row(netsnmp_request_info *request)
                                                                TABLE_DATA_ROW);
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_EXTRACT_TABLE_ROW_DATA
 /** extracts the data from the row being accessed passed from the
  * table_data helper */
 void           *
@@ -685,7 +723,9 @@ netsnmp_extract_table_row_data(netsnmp_request_info *request)
     else
         return NULL;
 }
+#endif /* NETSNMP_FEATURE_REMOVE_EXTRACT_TABLE_ROW_DATA */
 
+#ifndef NETSNMP_FEATURE_REMOVE_INSERT_TABLE_ROW
 /** inserts a newly created table_data row into a request */
 void
 netsnmp_insert_table_row(netsnmp_request_info *request,
@@ -750,6 +790,7 @@ netsnmp_insert_table_row(netsnmp_request_info *request,
         }
     }
 }
+#endif /* NETSNMP_FEATURE_REMOVE_INSERT_TABLE_ROW */
 
 /* builds a result given a row, a varbind to set and the data */
 int
@@ -861,11 +902,13 @@ netsnmp_table_data_num_rows(netsnmp_table_data *table)
      * Generic API - mostly renamed wrappers
      * ===================================== */
 
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_DATA_ROW_FIRST
 netsnmp_table_row *
 netsnmp_table_data_row_first(netsnmp_table_data *table)
 {
     return netsnmp_table_data_get_first_row(table);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_DATA_ROW_FIRST */
 
 netsnmp_table_row *
 netsnmp_table_data_row_get(  netsnmp_table_data *table,
@@ -930,11 +973,13 @@ netsnmp_table_data_row_next_byidx(netsnmp_table_data    *table,
     return netsnmp_table_data_row_next_byoid(table, instance, len);
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_DATA_ROW_COUNT
 int
 netsnmp_table_data_row_count(netsnmp_table_data *table)
 {
     return netsnmp_table_data_num_rows(table);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_DATA_ROW_COUNT */
 
 
 /* ==================================
@@ -944,6 +989,7 @@ netsnmp_table_data_row_count(netsnmp_table_data *table)
  *
  * ================================== */
 
+#ifndef NETSNMP_FEATURE_REMOVE_TABLE_DATA_ROW_OPERATIONS
 void *
 netsnmp_table_data_entry_first(netsnmp_table_data *table)
 {
@@ -1003,6 +1049,7 @@ netsnmp_table_data_entry_next_byoid(netsnmp_table_data *table,
         netsnmp_table_data_row_next_byoid(table, instance, len);
     return (row ? row->data : NULL );
 }
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_DATA_ROW_OPERATIONS */
 
     /* =====================================
      * Generic API - mostly renamed wrappers
@@ -1014,5 +1061,9 @@ netsnmp_table_data_entry_next_byoid(netsnmp_table_data *table,
  *
  * ================================== */
 
+#else /* NETSNMP_FEATURE_REMOVE_TABLE_DATA */
+netsnmp_feature_unused(table_data);
+#endif /* NETSNMP_FEATURE_REMOVE_TABLE_DATA */
+
 /** @} 
  */
