@@ -1,4 +1,5 @@
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include <net-snmp/agent/scalar.h>
@@ -12,6 +13,11 @@
 #include <net-snmp/library/snmp_logging.h>
 #include "agent/nsLogging.h"
 
+netsnmp_feature_require(logging_external)
+#ifndef NETSNMP_NO_WRITE_SUPPORT
+netsnmp_feature_require(table_iterator_insert_context)
+#endif /* NETSNMP_NO_WRITE_SUPPORT */
+
 /*
  * OID and columns for the logging table.
  */
@@ -58,7 +64,7 @@ init_nsLogging(void)
     /*
      * .... and register the table with the agent.
      */
-    netsnmp_register_table_iterator(
+    netsnmp_register_table_iterator2(
         netsnmp_create_handler_registration(
             "tzLoggingTable", handle_nsLoggingTable,
             nsLoggingTable_oid, OID_LENGTH(nsLoggingTable_oid),
@@ -186,6 +192,7 @@ handle_nsLoggingTable(netsnmp_mib_handler *handler,
 	break;
 
 
+#ifndef NETSNMP_NO_WRITE_SUPPORT
     case MODE_SET_RESERVE1:
         for (request=requests; request; request=request->next) {
             if ( request->status != 0 ) {
@@ -356,7 +363,7 @@ handle_nsLoggingTable(netsnmp_mib_handler *handler,
 		switch ( *request->requestvb->val.integer ) {
                 case RS_ACTIVE:
                 case RS_CREATEANDGO:
-                    if ( !logh->type ) {
+                    if ( !logh || !logh->type ) {
                         netsnmp_set_request_error(reqinfo, request,
                                                   SNMP_ERR_INCONSISTENTVALUE);
                         return SNMP_ERR_INCONSISTENTVALUE;
@@ -458,6 +465,7 @@ handle_nsLoggingTable(netsnmp_mib_handler *handler,
 	    }
 	}
 	break;
+#endif /* !NETSNMP_NO_WRITE_SUPPORT */
     }
 
     return SNMP_ERR_NOERROR;
