@@ -4,6 +4,7 @@
  */
 
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 
@@ -14,6 +15,8 @@
 #include "disman/expr/expObject.h"
 #include "disman/expr/expExpressionConf.h"
 
+netsnmp_feature_require(iquery)
+
 /* Initializes the expExpressionConf module */
 void
 init_expExpressionConf(void)
