@@ -99,11 +99,12 @@ int netsnmp_cpu_arch_load( netsnmp_cache *cache, void *magic ) {
     static char *buff  = NULL;
     static int   bsize = 0;
     static int   first = 1;
-    static int   has_cpu_26 = 1;
+    static int   num_cpuline_elem = 0;
     int          bytes_read, statfd, i;
     char        *b1, *b2;
     unsigned long long cusell = 0, cicell = 0, csysll = 0, cidell = 0,
-                       ciowll = 0, cirqll = 0, csoftll = 0;
+                       ciowll = 0, cirqll = 0, csoftll = 0, cstealll = 0,
+                       cguestll = 0, cguest_nicell = 0;
     netsnmp_cpu_info* cpu;
 
     if ((statfd = open(STAT_FILE, O_RDONLY, 0)) == -1) {
@@ -120,6 +121,10 @@ int netsnmp_cpu_arch_load( netsnmp_cache *cache, void *magic ) {
         DEBUGMSGTL(("cpu", "/proc/stat buffer increased to %d\n", bsize));
         close(statfd);
         statfd = open(STAT_FILE, O_RDONLY, 0);
+        if (statfd == -1) {
+            snmp_log_perror(STAT_FILE);
+            return -1;
+	}
     }
     close(statfd);
 
@@ -152,22 +157,33 @@ int netsnmp_cpu_arch_load( netsnmp_cache *cache, void *magic ) {
             b1 = b2+5; /* Skip "cpuN " */
         }
 
-	if (!has_cpu_26 ||
-            sscanf(b1, "%llu %llu %llu %llu %llu %llu %llu", &cusell,
-                   &cicell, &csysll, &cidell, &ciowll, &cirqll, &csoftll) != 7) {
-	    has_cpu_26 = 0;
-	    sscanf(b1, "%llu %llu %llu %llu", &cusell, &cicell, &csysll,
-                   &cidell);
+        num_cpuline_elem = sscanf(b1, "%llu %llu %llu %llu %llu %llu %llu %llu %llu %llu",
+         &cusell, &cicell, &csysll, &cidell, &ciowll, &cirqll, &csoftll, &cstealll, &cguestll, &cguest_nicell);
+        DEBUGMSGTL(("cpu", "/proc/stat cpu line number of elements: %i\n", num_cpuline_elem));
 
-	} else {
-            cpu->wait_ticks   = (unsigned long)ciowll;
-            cpu->intrpt_ticks = (unsigned long)cirqll;
-            cpu->sirq_ticks   = (unsigned long)csoftll;
- 	}
-        cpu->user_ticks = (unsigned long)cusell;
-        cpu->nice_ticks = (unsigned long)cicell;
-        cpu->sys_ticks  = (unsigned long)csysll;
-        cpu->idle_ticks = (unsigned long)cidell;
+        /* kernel 2.6.33 and above */
+        if (num_cpuline_elem == 10) {
+            cpu->guestnice_ticks = (unsigned long long)cguest_nicell;
+        }
+        /* kernel 2.6.24 and above */
+        if (num_cpuline_elem >= 9) {
+            cpu->guest_ticks = (unsigned long long)cguestll;
+        }
+        /* kernel 2.6.11 and above */
+        if (num_cpuline_elem >= 8) {
+            cpu->steal_ticks = (unsigned long long)cstealll;
+        }
+        /* kernel 2.6 */
+        if (num_cpuline_elem >= 5) {
+            cpu->wait_ticks   = (unsigned long long)ciowll;
+            cpu->intrpt_ticks = (unsigned long long)cirqll;
+            cpu->sirq_ticks   = (unsigned long long)csoftll;
+        }
+        /* rest */
+        cpu->user_ticks = (unsigned long long)cusell;
+        cpu->nice_ticks = (unsigned long long)cicell;
+        cpu->sys_ticks  = (unsigned long long)csysll;
+        cpu->idle_ticks = (unsigned long long)cidell;
     }
     if ( b1 == buff ) {
 	if (first)
@@ -205,7 +221,12 @@ void _cpu_load_swap_etc( char *buff, netsnmp_cpu_info *cpu ) {
     unsigned long long pin, pout, swpin, swpout;
     unsigned long long itot, iticks, ctx;
 
-    if (has_vmstat && (vmstatfd = open(VMSTAT_FILE, O_RDONLY, 0)) != -1) {
+    if (has_vmstat) {
+      vmstatfd = open(VMSTAT_FILE, O_RDONLY, 0);
+      if (vmstatfd == -1 ) {
+            snmp_log(LOG_ERR, "cannot open %s\n", VMSTAT_FILE);
+            has_vmstat = 0;
+      } else {
         if (vmbsize == 0) {
 	    vmbsize = getpagesize()-1;
 	    vmbuff = (char*)malloc(vmbsize+1);
@@ -215,22 +236,25 @@ void _cpu_load_swap_etc( char *buff, netsnmp_cpu_info *cpu ) {
 	    vmbuff = (char*)realloc(vmbuff, vmbsize+1);
 	    close(vmstatfd);
 	    vmstatfd = open(VMSTAT_FILE, O_RDONLY, 0);
+	    if (vmstatfd == -1) {
+                snmp_log_perror("cannot open " VMSTAT_FILE);
+                return;
+	    }
         }
         close(vmstatfd);
         if ( bytes_read < 0 ) {
-            snmp_log_perror(STAT_FILE "read error");
+            snmp_log_perror(VMSTAT_FILE "read error");
             return;
         }
         vmbuff[bytes_read] = '\0';
+      }
     }
-    else
-        has_vmstat = 0;
 
     if (has_vmstat) {
 	b = strstr(vmbuff, "pgpgin ");
 	if (b) {
 	    sscanf(b, "pgpgin %llu", &pin);
-            cpu->pageIn  = (unsigned long)pin*2;  /* ??? */
+            cpu->pageIn  = (unsigned long long)pin*2;  /* ??? */
 	} else {
 	    if (first)
 		snmp_log(LOG_ERR, "No pgpgin line in %s\n", VMSTAT_FILE);
@@ -239,7 +263,7 @@ void _cpu_load_swap_etc( char *buff, netsnmp_cpu_info *cpu ) {
 	b = strstr(vmbuff, "pgpgout ");
 	if (b) {
 	    sscanf(b, "pgpgout %llu", &pout);
-            cpu->pageOut = (unsigned long)pout*2;  /* ??? */
+            cpu->pageOut = (unsigned long long)pout*2;  /* ??? */
 	} else {
 	    if (first)
 		snmp_log(LOG_ERR, "No pgpgout line in %s\n", VMSTAT_FILE);
@@ -248,7 +272,7 @@ void _cpu_load_swap_etc( char *buff, netsnmp_cpu_info *cpu ) {
 	b = strstr(vmbuff, "pswpin ");
 	if (b) {
 	    sscanf(b, "pswpin %llu", &swpin);
-            cpu->swapIn  = (unsigned long)swpin;
+            cpu->swapIn  = (unsigned long long)swpin;
 	} else {
 	    if (first)
 		snmp_log(LOG_ERR, "No pswpin line in %s\n", VMSTAT_FILE);
@@ -257,7 +281,7 @@ void _cpu_load_swap_etc( char *buff, netsnmp_cpu_info *cpu ) {
 	b = strstr(vmbuff, "pswpout ");
 	if (b) {
 	    sscanf(b, "pswpout %llu", &swpout);
-            cpu->swapOut = (unsigned long)swpout;
+            cpu->swapOut = (unsigned long long)swpout;
 	} else {
 	    if (first)
 		snmp_log(LOG_ERR, "No pswpout line in %s\n", VMSTAT_FILE);
@@ -268,8 +292,8 @@ void _cpu_load_swap_etc( char *buff, netsnmp_cpu_info *cpu ) {
 	b = strstr(buff, "page ");
 	if (b) {
 	    sscanf(b, "page %llu %llu", &pin, &pout);
-            cpu->pageIn  = (unsigned long)pin;
-            cpu->pageOut = (unsigned long)pout;
+            cpu->pageIn  = (unsigned long long)pin;
+            cpu->pageOut = (unsigned long long)pout;
 	} else {
 	    if (first)
 		snmp_log(LOG_ERR, "No page line in %s\n", STAT_FILE);
@@ -278,8 +302,8 @@ void _cpu_load_swap_etc( char *buff, netsnmp_cpu_info *cpu ) {
 	b = strstr(buff, "swap ");
 	if (b) {
 	    sscanf(b, "swap %llu %llu", &swpin, &swpout);
-            cpu->swapIn  = (unsigned long)swpin;
-            cpu->swapOut = (unsigned long)swpout;
+            cpu->swapIn  = (unsigned long long)swpin;
+            cpu->swapOut = (unsigned long long)swpout;
 	} else {
 	    if (first)
 		snmp_log(LOG_ERR, "No swap line in %s\n", STAT_FILE);
@@ -290,7 +314,7 @@ void _cpu_load_swap_etc( char *buff, netsnmp_cpu_info *cpu ) {
     b = strstr(buff, "intr ");
     if (b) {
 	sscanf(b, "intr %llu %llu", &itot, &iticks);
-        cpu->nInterrupts = (unsigned long)itot;
+        cpu->nInterrupts = (unsigned long long)itot;
         /* iticks not used? */
     } else {
 	if (first)
@@ -299,10 +323,11 @@ void _cpu_load_swap_etc( char *buff, netsnmp_cpu_info *cpu ) {
     b = strstr(buff, "ctxt ");
     if (b) {
 	sscanf(b, "ctxt %llu", &ctx);
-        cpu->nCtxSwitches = (unsigned long)ctx;
+        cpu->nCtxSwitches = (unsigned long long)ctx;
     } else {
 	if (first)
 	    snmp_log(LOG_ERR, "No ctxt line in %s\n", STAT_FILE);
     }
+    first = 0;
 }
 
