@@ -355,7 +355,7 @@ parse_disk_config(const char *token, char *cptr)
             di_curr->item_details = (void *) 0;
             name++;
         } else if (*name == '[') {
-            d_set = (details_set *) malloc(sizeof(details_set));
+            d_set = (details_set *) calloc(sizeof(details_set), 1);
             if (!d_set) {
                 config_perror("Out of memory");
                 SNMP_FREE(d_new);
@@ -364,8 +364,6 @@ parse_disk_config(const char *token, char *cptr)
                 SNMP_FREE(d_str);
                 return;
             }
-            for (i = 0; i < sizeof(details_set); i++)
-                (*d_set)[i] = (unsigned char) 0;
             name++;
             if (*name == '^' || *name == '!') {
                 neg = 1;
@@ -904,16 +902,14 @@ Get_HR_Disk_Label(char *string, size_t str_len, const char *devfull)
 
     sess_ref = DASessionCreate( NULL );
     if (NULL == sess_ref) {
-        strncpy(string, devfull, str_len);
-        string[str_len-1] = 0;
+        strlcpy(string, devfull, str_len);
         return -1;
     }
 
     disk = DADiskCreateFromBSDName( NULL, sess_ref, devfull );
     if (NULL == disk) {
         CFRelease(sess_ref);
-        strncpy(string, devfull, str_len);
-        string[str_len-1] = 0;
+        strlcpy(string, devfull, str_len);
         return -1;
     }
 
@@ -924,7 +920,7 @@ Get_HR_Disk_Label(char *string, size_t str_len, const char *devfull)
                  devfull);
         CFRelease(disk);
         CFRelease(sess_ref);
-        strncpy(string, devfull, str_len);
+        strlcpy(string, devfull, str_len);
         return -1;
     }
 
@@ -932,14 +928,12 @@ Get_HR_Disk_Label(char *string, size_t str_len, const char *devfull)
     str_ref = (CFStringRef)
         CFDictionaryGetValue(desc, kDADiskDescriptionMediaNameKey);
     if (str_ref) {
-        strncpy(string, CFStringGetCStringPtr(str_ref, sys_encoding),
+        strlcpy(string, CFStringGetCStringPtr(str_ref, sys_encoding),
                 str_len);
-        string[str_len-1] = 0;
         DEBUGMSGTL(("verbose:diskmgr:darwin", " name %s\n", string));
     }
     else {
-        strncpy(string, devfull, str_len);
-        string[str_len-1] = 0;
+        strlcpy(string, devfull, str_len);
     }
     
     CFRelease(disk);
@@ -984,27 +978,21 @@ static void
 Save_HR_Disk_General(void)
 {
 #ifdef DIOC_DESCRIBE
-    strncpy(HRD_savedModel, HRD_info.model_num, sizeof(HRD_savedModel)-1);
-    HRD_savedModel[ sizeof(HRD_savedModel)-1 ] = 0;
+    strlcpy(HRD_savedModel, HRD_info.model_num, sizeof(HRD_savedModel));
 #endif
 #ifdef DKIOCINFO
-    strncpy(HRD_savedModel, HRD_info.dki_dname, sizeof(HRD_savedModel)-1);
-    HRD_savedModel[ sizeof(HRD_savedModel)-1 ] = 0;
+    strlcpy(HRD_savedModel, HRD_info.dki_dname, sizeof(HRD_savedModel));
 #endif
 #ifdef HAVE_LINUX_HDREG_H
-    strncpy(HRD_savedModel, (const char *) HRD_info.model,
-                    sizeof(HRD_savedModel)-1);
-    HRD_savedModel[ sizeof(HRD_savedModel)-1 ] = 0;
+    strlcpy(HRD_savedModel, (const char *) HRD_info.model,
+            sizeof(HRD_savedModel));
 #endif
 #ifdef DIOCGDINFO
-    strncpy(HRD_savedModel, dktypenames[HRD_info.d_type],
-                    sizeof(HRD_savedModel)-1);
-    HRD_savedModel[ sizeof(HRD_savedModel)-1 ] = 0;
+    strlcpy(HRD_savedModel, dktypenames[HRD_info.d_type],
+            sizeof(HRD_savedModel));
 #endif
 #ifdef darwin
-    strncpy(HRD_savedModel, HRD_model,
-                    sizeof(HRD_savedModel)-1);
-    HRD_savedModel[ sizeof(HRD_savedModel)-1 ] = 0;
+    strlcpy(HRD_savedModel, HRD_model, sizeof(HRD_savedModel));
 #endif
 }
 
@@ -1149,9 +1137,8 @@ Query_Disk(int fd, const char *devfull)
     str_ref = (CFStringRef)
         CFDictionaryGetValue(desc, kDADiskDescriptionDeviceModelKey);
     if (str_ref) {
-        strncpy(HRD_model, CFStringGetCStringPtr(str_ref, sys_encoding),
+        strlcpy(HRD_model, CFStringGetCStringPtr(str_ref, sys_encoding),
                 sizeof(HRD_model));
-        HRD_savedModel[ sizeof(HRD_savedModel)-1 ] = 0;
         DEBUGMSGTL(("verbose:diskmgr:darwin", " model %s\n", HRD_model));
     }
     else
