@@ -1,18 +1,28 @@
 /*
  *  Interface MIB architecture support
  *
- * $Id: interface.c 19018 2010-06-16 21:34:42Z dts12 $
+ * $Id$
  */
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <net-snmp/net-snmp-includes.h>
-#include "mibII/mibII_common.h"
-#include "if-mib/ifTable/ifTable_constants.h"
-#include "if-mib/data_access/interface.h"
 
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include <net-snmp/library/snmp_enum.h>
 #include <net-snmp/data_access/interface.h>
 
+#include "mibII/mibII_common.h"
+#include "if-mib/ifTable/ifTable.h"
+#include "if-mib/data_access/interface.h"
+
+netsnmp_feature_child_of(interface_all, libnetsnmpmibs)
+netsnmp_feature_child_of(interface, interface_all)
+netsnmp_feature_child_of(interface_access_entry_set_admin_status, interface_all)
+netsnmp_feature_child_of(interface_legacy, interface_all)
+
+#ifdef NETSNMP_FEATURE_REQUIRE_INTERFACE_ACCESS_ENTRY_SET_ADMIN_STATUS
+netsnmp_feature_require(interface_arch_set_admin_status)
+#endif /* NETSNMP_FEATURE_REQUIRE_INTERFACE_ACCESS_ENTRY_SET_ADMIN_STATUS */
 
 /**---------------------------------------------------------------------*/
 /*
@@ -337,6 +347,7 @@ netsnmp_access_interface_entry_free(netsnmp_interface_entry * entry)
     free(entry);
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_INTERFACE_LEGACY
 /*
  * Blech - backwards compatible mibII/interfaces style interface
  * functions, so we don't have to update older modules to use
@@ -346,42 +357,60 @@ netsnmp_access_interface_entry_free(netsnmp_interface_entry * entry)
     ! defined( NETSNMP_NO_BACKWARDS_COMPATABILITY )
 
 static netsnmp_iterator *it = NULL;
-static netsnmp_container *c = NULL;
-static netsnmp_interface_entry *e = NULL;
+static ifTable_rowreq_ctx *row = NULL;
 
 /**
- * 
+ * Setup an iterator for scanning the interfaces using the cached entry
+ * from if-mib/ifTable.
  */
 void
 Interface_Scan_Init(void)
 {
-    /*
-     * ifTable container shouldn't change, so we shouldn' have to
-     * re-fetch it every time.
-     */
-    if (NULL != c)
-        netsnmp_access_interface_container_free(c, 0);
-
-    c = netsnmp_access_interface_container_load(NULL, 0);
+    netsnmp_container *cont = NULL;
+    netsnmp_cache *cache    = NULL; 
     
-    if (NULL != c) {
+    cache = netsnmp_cache_find_by_oid(ifTable_oid, ifTable_oid_size);
+    if (NULL != cache) {
+        netsnmp_cache_check_and_reload(cache);
+        cont = (netsnmp_container*) cache->magic;
+    }
+    
+    if (NULL != cont) {
         if (NULL != it)
             ITERATOR_RELEASE(it);
     
-        it = CONTAINER_ITERATOR(c);
+        it = CONTAINER_ITERATOR(cont);
     }
    
     if (NULL != it)
-        e = (netsnmp_interface_entry*)ITERATOR_FIRST(it);
+        row = (ifTable_rowreq_ctx*)ITERATOR_FIRST(it);
 }
 
 int
 Interface_Scan_Next(short *index, char *name, netsnmp_interface_entry **entry,
                     void *dc)
 {
-    if (NULL == e)
-        return 0;
+    int returnIndex = 0;
+    int ret;
+    if (index)
+        returnIndex = *index;
 
+    ret = Interface_Scan_NextInt( &returnIndex, name, entry, dc );
+    if (index)
+        *index = (returnIndex & 0x8fff);
+    return ret;
+}
+
+int
+Interface_Scan_NextInt(int *index, char *name, netsnmp_interface_entry **entry,
+                    void *dc)
+{
+    netsnmp_interface_entry* e = NULL;
+
+    if (NULL == row)
+        return 0;
+    
+    e = row->data.ifentry;
     if(index)
         *index = e->index;
 
@@ -391,13 +420,14 @@ Interface_Scan_Next(short *index, char *name, netsnmp_interface_entry **entry,
     if (entry)
         *entry = e;
 
-    e = (netsnmp_interface_entry*)ITERATOR_NEXT(it);
+    row = (ifTable_rowreq_ctx*) ITERATOR_NEXT(it);
 
     return 1;
 }
 #endif /* NETSNMP_NO_BACKWARDS_COMPATABILITY */
+#endif /* NETSNMP_FEATURE_REMOVE_INTERFACE_LEGACY */
 
-
+#ifndef NETSNMP_FEATURE_REMOVE_INTERFACE_ACCESS_ENTRY_SET_ADMIN_STATUS
 /**
  *
  * @retval 0   : success
@@ -426,6 +456,7 @@ netsnmp_access_interface_entry_set_admin_status(netsnmp_interface_entry * entry,
     return rc;
 }
 #endif
+#endif /* NETSNMP_FEATURE_REMOVE_INTERFACE_ACCESS_ENTRY_SET_ADMIN_STATUS */
 
 /**---------------------------------------------------------------------*/
 /*
@@ -512,14 +543,14 @@ netsnmp_access_interface_entry_update_stats(netsnmp_interface_entry * prev_vals,
         if (NULL == prev_vals->old_stats) {
             return -2;
         }
-        memcpy(prev_vals->old_stats, &prev_vals->stats, sizeof(new_vals->stats));
+        memcpy(prev_vals->old_stats, &prev_vals->stats, sizeof(prev_vals->stats));
     }
 
         if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.ibytes,
                                        &new_vals->stats.ibytes,
                                        &prev_vals->old_stats->ibytes,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
+            DEBUGMSGTL(("access:interface",
                     "Error expanding ifHCInOctets to 64bits\n"));
 
         if (new_vals->ns_flags & NETSNMP_INTERFACE_FLAGS_CALCULATE_UCAST) {
@@ -527,14 +558,14 @@ netsnmp_access_interface_entry_update_stats(netsnmp_interface_entry * prev_vals,
                                            &new_vals->stats.iall,
                                            &prev_vals->old_stats->iall,
                                            &need_wrap_check))
-                NETSNMP_LOGONCE((LOG_ERR,
+                DEBUGMSGTL(("access:interface",
                         "Error expanding packet count to 64bits\n"));
         } else {
             if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.iucast,
                                            &new_vals->stats.iucast,
                                            &prev_vals->old_stats->iucast,
                                            &need_wrap_check))
-                NETSNMP_LOGONCE((LOG_ERR,
+                DEBUGMSGTL(("access:interface",
                         "Error expanding ifHCInUcastPkts to 64bits\n"));
         }
 
@@ -542,49 +573,49 @@ netsnmp_access_interface_entry_update_stats(netsnmp_interface_entry * prev_vals,
                                        &new_vals->stats.iucast,
                                        &prev_vals->old_stats->iucast,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
+            DEBUGMSGTL(("access:interface",
                     "Error expanding ifHCInUcastPkts to 64bits\n"));
 
         if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.imcast,
                                        &new_vals->stats.imcast,
                                        &prev_vals->old_stats->imcast,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
+            DEBUGMSGTL(("access:interface",
                     "Error expanding ifHCInMulticastPkts to 64bits\n"));
 
         if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.ibcast,
                                        &new_vals->stats.ibcast,
                                        &prev_vals->old_stats->ibcast,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
+            DEBUGMSGTL(("access:interface",
                     "Error expanding ifHCInBroadcastPkts to 64bits\n"));
 
         if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.obytes,
                                        &new_vals->stats.obytes,
                                        &prev_vals->old_stats->obytes,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
+            DEBUGMSGTL(("access:interface",
                     "Error expanding ifHCOutOctets to 64bits\n"));
 
         if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.oucast,
                                        &new_vals->stats.oucast,
                                        &prev_vals->old_stats->oucast,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
+            DEBUGMSGTL(("access:interface",
                     "Error expanding ifHCOutUcastPkts to 64bits\n"));
 
         if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.omcast,
                                        &new_vals->stats.omcast,
                                        &prev_vals->old_stats->omcast,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
+            DEBUGMSGTL(("access:interface",
                     "Error expanding ifHCOutMulticastPkts to 64bits\n"));
 
         if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.obcast,
                                        &new_vals->stats.obcast,
                                        &prev_vals->old_stats->obcast,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
+            DEBUGMSGTL(("access:interface",
                     "Error expanding ifHCOutBroadcastPkts to 64bits\n"));
 
     /*
@@ -666,8 +697,7 @@ netsnmp_access_interface_entry_copy(netsnmp_interface_entry * lhs,
        (0 == strcmp(lhs->descr, rhs->descr)))
         ;
     else {
-        if (NULL != lhs->descr)
-            SNMP_FREE(lhs->descr);
+        SNMP_FREE(lhs->descr);
         if (rhs->descr) {
             lhs->descr = strdup(rhs->descr);
             if(NULL == lhs->descr)
@@ -695,8 +725,7 @@ netsnmp_access_interface_entry_copy(netsnmp_interface_entry * lhs,
         if(rhs->paddr_len)
             memcpy(lhs->paddr,rhs->paddr,rhs->paddr_len);
     } else {
-        if (NULL != lhs->paddr)
-            SNMP_FREE(lhs->paddr);
+        SNMP_FREE(lhs->paddr);
         if (rhs->paddr) {
             lhs->paddr = (char*)malloc(rhs->paddr_len);
             if(NULL == lhs->paddr)
