@@ -21,6 +21,11 @@ _netsnmp_ioctl_ipaddress_set_v4(netsnmp_ipaddress_entry * entry);
 int
 _netsnmp_ioctl_ipaddress_remove_v4(netsnmp_ipaddress_entry * entry);
 
+int
+_netsnmp_ioctl_ipaddress_set_v6(netsnmp_ipaddress_entry * entry);
+int
+_netsnmp_ioctl_ipaddress_remove_v6(netsnmp_ipaddress_entry * entry);
+
 int
 netsnmp_access_ipaddress_ioctl_get_interface_count(int sd, struct ifconf * ifc);
 
@@ -50,6 +55,8 @@ netsnmp_ioctl_ipaddress_extras_get(netsnmp_ipaddress_entry *entry);
 
 int
 _netsnmp_ioctl_ipaddress_delete_v4(netsnmp_ipaddress_entry * entry);
+int
+_netsnmp_ioctl_ipaddress_delete_v6(netsnmp_ipaddress_entry * entry);
 
 #ifdef __cplusplus
 }
