@@ -1,7 +1,7 @@
 /*
  *  ipSystemStatsTable and ipIfStatsTable MIB architecture support
  *
- * $Id: systemstats_common.c 18993 2010-06-16 03:28:07Z rstory $
+ * $Id$
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -314,6 +314,7 @@ netsnmp_access_systemstats_entry_update_stats(netsnmp_systemstats_entry * prev_v
         if (NULL == prev_vals->old_stats) {
             return -2;
         }
+        memcpy(&prev_vals->stats, &new_vals->stats, sizeof(new_vals->stats));
     }
     else {
         /*
@@ -332,173 +333,215 @@ netsnmp_access_systemstats_entry_update_stats(netsnmp_systemstats_entry * prev_v
         /*
          * update 64bit counters
          */
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCInNoRoutes,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCINNOROUTES])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCInNoRoutes,
                                        &new_vals->stats.HCInNoRoutes,
                                        &prev_vals->old_stats->HCInNoRoutes,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCInNoRoutes to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCInNoRoutes to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCOutNoRoutes,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCOUTNOROUTES])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCOutNoRoutes,
                                        &new_vals->stats.HCOutNoRoutes,
                                        &prev_vals->old_stats->HCOutNoRoutes,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCOutNoRoutes to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCOutNoRoutes to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCOutDiscards,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCOUTDISCARDS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCOutDiscards,
                                        &new_vals->stats.HCOutDiscards,
                                        &prev_vals->old_stats->HCOutDiscards,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCOutDiscards to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCOutDiscards to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCOutFragReqds,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCOUTFRAGREQDS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCOutFragReqds,
                                        &new_vals->stats.HCOutFragReqds,
                                        &prev_vals->old_stats->HCOutFragReqds,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCOutFragReqds to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCOutFragReqds to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCOutFragOKs,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCOUTFRAGOKS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCOutFragOKs,
                                        &new_vals->stats.HCOutFragOKs,
                                        &prev_vals->old_stats->HCOutFragOKs,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCOutFragOKs to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCOutFragOKs to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCOutFragFails,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCOUTFRAGFAILS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCOutFragFails,
                                        &new_vals->stats.HCOutFragFails,
                                        &prev_vals->old_stats->HCOutFragFails,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCOutFragFails to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCOutFragFails to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCOutFragCreates,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCOUTFRAGCREATES])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCOutFragCreates,
                                        &new_vals->stats.HCOutFragCreates,
                                        &prev_vals->old_stats->HCOutFragCreates,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCOutFragCreates to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCOutFragCreates to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCInReceives,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCINRECEIVES])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCInReceives,
                                        &new_vals->stats.HCInReceives,
                                        &prev_vals->old_stats->HCInReceives,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCInReceives to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCInReceives to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCInOctets,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCINOCTETS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCInOctets,
                                        &new_vals->stats.HCInOctets,
                                        &prev_vals->old_stats->HCInOctets,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCInOctets to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCInOctets to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCInForwDatagrams,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCINFORWDATAGRAMS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCInForwDatagrams,
                                        &new_vals->stats.HCInForwDatagrams,
                                        &prev_vals->old_stats->HCInForwDatagrams,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCInForwDatagrams to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCInForwDatagrams to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCInDelivers,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCINDELIVERS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCInDelivers,
                                        &new_vals->stats.HCInDelivers,
                                        &prev_vals->old_stats->HCInDelivers,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCInDelivers to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCInDelivers to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCOutRequests,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCOUTREQUESTS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCOutRequests,
                                        &new_vals->stats.HCOutRequests,
                                        &prev_vals->old_stats->HCOutRequests,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCOutRequests to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCOutRequests to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCOutForwDatagrams,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCOUTFORWDATAGRAMS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCOutForwDatagrams,
                                        &new_vals->stats.HCOutForwDatagrams,
                                        &prev_vals->old_stats->HCOutForwDatagrams,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCOutForwDatagrams to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCOutForwDatagrams to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCOutTransmits,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCOUTTRANSMITS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCOutTransmits,
                                        &new_vals->stats.HCOutTransmits,
                                        &prev_vals->old_stats->HCOutTransmits,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCOutTransmits to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCOutTransmits to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCOutOctets,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCOUTOCTETS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCOutOctets,
                                        &new_vals->stats.HCOutOctets,
                                        &prev_vals->old_stats->HCOutOctets,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCOutOctets to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCOutOctets to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCInMcastPkts,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCINMCASTPKTS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCInMcastPkts,
                                        &new_vals->stats.HCInMcastPkts,
                                        &prev_vals->old_stats->HCInMcastPkts,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCInMcastPkts to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCInMcastPkts to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCInMcastOctets,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCINMCASTOCTETS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCInMcastOctets,
                                        &new_vals->stats.HCInMcastOctets,
                                        &prev_vals->old_stats->HCInMcastOctets,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCInMcastOctets to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCInMcastOctets to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCOutMcastPkts,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCOUTMCASTPKTS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCOutMcastPkts,
                                        &new_vals->stats.HCOutMcastPkts,
                                        &prev_vals->old_stats->HCOutMcastPkts,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCOutMcastPkts to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCOutMcastPkts to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCOutMcastOctets,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCOUTMCASTOCTETS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCOutMcastOctets,
                                        &new_vals->stats.HCOutMcastOctets,
                                        &prev_vals->old_stats->HCOutMcastOctets,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCOutMcastOctets to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCOutMcastOctets to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCInBcastPkts,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCINBCASTPKTS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCInBcastPkts,
                                        &new_vals->stats.HCInBcastPkts,
                                        &prev_vals->old_stats->HCInBcastPkts,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCInBcastPkts to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCInBcastPkts to 64bits in %s\n",
+                        prev_vals->tableName));
 
-        if (0 != netsnmp_c64_check32_and_update(&prev_vals->stats.HCOutBcastPkts,
+        if (new_vals->stats.columnAvail[IPSYSTEMSTATSTABLE_HCOUTBCASTPKTS])
+            if (0 != netsnmp_c64_check32_and_update(
+                                       &prev_vals->stats.HCOutBcastPkts,
                                        &new_vals->stats.HCOutBcastPkts,
                                        &prev_vals->old_stats->HCOutBcastPkts,
                                        &need_wrap_check))
-            NETSNMP_LOGONCE((LOG_ERR,
-                    "Error expanding HCOutBcastPkts to 64bits in %s\n",
-                    prev_vals->tableName));
+                DEBUGMSGTL(("access:systemstats",
+                        "Error expanding HCOutBcastPkts to 64bits in %s\n",
+                        prev_vals->tableName));
     }
 
     /*
