@@ -96,6 +96,9 @@
 #endif
 #ifdef linux
 #include "kernel_linux.h"
+#endif
+#ifdef NETBSD_STATS_VIA_SYSCTL
+#include "kernel_netbsd.h"
 #endif
 	/* or MIB_xxxCOUNTER_SYMBOL || hpux11 */
 #ifdef hpux
