@@ -4,6 +4,7 @@
  */
 
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <sys/types.h>
 #include <ctype.h>
 
@@ -68,6 +69,8 @@
 #include "mibdefs.h"
 #include "snmpd.h"
 
+netsnmp_feature_require(snprint_objid)
+
 long            smux_long;
 u_long          smux_ulong;
 struct sockaddr_in smux_sa;
@@ -88,7 +91,7 @@ static u_char  *smux_parse(u_char *, oid *, size_t *, size_t *, u_char *);
 static u_char  *smux_parse_var(u_char *, size_t *, oid *, size_t *,
                                size_t *, u_char *);
 static void     smux_send_close(int, int);
-static void     smux_list_detach(smux_reg **, smux_reg **);
+static void     smux_list_detach(smux_reg **, smux_reg *);
 static void     smux_replace_active(smux_reg *, smux_reg *);
 static void     smux_peer_cleanup(int);
 static int      smux_auth_peer(oid *, size_t, char *, int);
@@ -142,6 +145,7 @@ smux_parse_peer_auth(const char *token, char *cptr)
     }
     if (nauths == SMUX_MAX_PEERS) {
 	config_perror("Too many smuxpeers");
+	free(aptr);
 	return;
     }
 
@@ -166,10 +170,8 @@ smux_parse_peer_auth(const char *token, char *cptr)
         /*
          * password 
          */
-        if (*password_cptr) {
-            strncpy(aptr->sa_passwd, password_cptr, SMUXMAXSTRLEN-1);
-            aptr->sa_passwd[SMUXMAXSTRLEN-1] = '\0';
-        }
+        if (*password_cptr)
+            strlcpy(aptr->sa_passwd, password_cptr, sizeof(aptr->sa_passwd));
     } else {
         /*
          * null passwords OK 
@@ -348,14 +350,13 @@ var_smux_write(int action,
     size_t          len;
     ssize_t         tmp_len;
     long            reqid, errsts, erridx;
-    u_char          var_type, *dataptr;
+    u_char          *dataptr;
 
     DEBUGMSGTL(("smux", "[var_smux_write] entering var_smux_write\n"));
 
     len = SMUXMAXPKTSIZE;
     reterr = SNMP_ERR_NOERROR;
     var_len = var_val_len;
-    var_type = var_val_type;
     name_length = name_len;
 
     /*
@@ -367,6 +368,11 @@ var_smux_write(int action,
             break;
     }
 
+    if (!rptr) {
+        DEBUGMSGTL(("smux", "[var_smux_write] unknown registration\n"));
+        return SNMP_ERR_GENERR;
+    }
+
     switch (action) {
     case RESERVE1:
         DEBUGMSGTL(("smux", "[var_smux_write] entering RESERVE1\n"));
@@ -411,7 +417,7 @@ var_smux_write(int action,
             return SNMP_ERR_GENERR;
         }
 
-        if (sendto(rptr->sr_fd, buf, len, 0, NULL, 0) < 0) {
+        if (sendto(rptr->sr_fd, (void *) buf, len, 0, NULL, 0) < 0) {
             DEBUGMSGTL(("smux", "[var_smux_write] send failed\n"));
             return SNMP_ERR_GENERR;
         }
@@ -420,7 +426,7 @@ var_smux_write(int action,
             /*
              * peek at what's received 
              */
-            if ((len = recvfrom(rptr->sr_fd, buf,
+            if ((len = recvfrom(rptr->sr_fd, (void *) buf,
                             SMUXMAXPKTSIZE, MSG_PEEK, NULL, NULL)) <= 0) {
                 if ((len == -1) && ((errno == EINTR) || (errno == EAGAIN)))
                 {
@@ -460,7 +466,7 @@ var_smux_write(int action,
             do
             {
                len = tmp_len;
-               len = recvfrom(rptr->sr_fd, buf, len, 0, NULL, NULL);
+               len = recvfrom(rptr->sr_fd, (void *) buf, len, 0, NULL, NULL);
             }
             while((len == -1) && ((errno == EINTR) || (errno == EAGAIN)));
 
@@ -544,7 +550,7 @@ var_smux_write(int action,
                         "[var_smux_write] entering FREE - sending Commit \n"));
         }
 
-        if ((sendto(rptr->sr_fd, sout, 3, 0, NULL, 0)) < 0) {
+        if ((sendto(rptr->sr_fd, (void *) sout, 3, 0, NULL, 0)) < 0) {
             DEBUGMSGTL(("smux",
                         "[var_smux_write] send rollback/commit failed\n"));
             return SNMP_ERR_GENERR;
@@ -1047,7 +1053,7 @@ smux_rreq_process(int sd, u_char * ptr, size_t * len)
                 /*
                  * no replacement found 
                  */
-                smux_list_detach(&ActiveRegs, &rptr);
+                smux_list_detach(&ActiveRegs, rptr);
                 free(rptr);
             }
             smux_send_rrsp(sd, rpriority);
@@ -1061,7 +1067,7 @@ smux_rreq_process(int sd, u_char * ptr, size_t * len)
                             priority);
         if (rptr) {
             rpriority = rptr->sr_priority;
-            smux_list_detach(&PassiveRegs, &rptr);
+            smux_list_detach(&PassiveRegs, rptr);
             free(rptr);
             smux_send_rrsp(sd, rpriority);
             return ptr;
@@ -1102,8 +1108,8 @@ smux_rreq_process(int sd, u_char * ptr, size_t * len)
                 snmp_oid_compare(oid_name, oid_name_len, rptr->sr_name,
                                  rptr->sr_name_len);
             if (result == 0) {
-                if ((oid_name_len == rptr->sr_name_len)) {
-                    if ((nrptr->sr_priority == -1)) {
+                if (oid_name_len == rptr->sr_name_len) {
+                    if (nrptr->sr_priority == -1) {
                         nrptr->sr_priority = rptr->sr_priority;
                         do {
                             nrptr->sr_priority++;
@@ -1152,9 +1158,17 @@ smux_rreq_process(int sd, u_char * ptr, size_t * len)
         if (nrptr->sr_priority == -1)
             nrptr->sr_priority = 0;
         smux_list_add(&ActiveRegs, nrptr);
-        register_mib("smux", (struct variable *)
-                     smux_variables, sizeof(struct variable2),
-                     1, nrptr->sr_name, nrptr->sr_name_len);
+        if (register_mib("smux", (struct variable *)
+                             smux_variables, sizeof(struct variable2),
+                             1, nrptr->sr_name, nrptr->sr_name_len)
+                     != SNMPERR_SUCCESS) {
+		DEBUGMSGTL(("smux", "[smux_rreq_process] Failed to register subtree\n"));
+		smux_list_detach(&ActiveRegs, nrptr);
+		free(nrptr);
+		smux_send_rrsp(sd, -1);
+		return NULL;
+	}
+
       done:
         smux_send_rrsp(sd, nrptr->sr_priority);
         return ptr;
@@ -1200,10 +1214,10 @@ smux_find_match(smux_reg * regs, int sd, oid * oid_name,
 static void
 smux_replace_active(smux_reg * actptr, smux_reg * pasptr)
 {
-    smux_list_detach(&ActiveRegs, &actptr);
+    smux_list_detach(&ActiveRegs, actptr);
     unregister_mib(actptr->sr_name, actptr->sr_name_len);
 
-    smux_list_detach(&PassiveRegs, &pasptr);
+    smux_list_detach(&PassiveRegs, pasptr);
     (void) smux_list_add(&ActiveRegs, pasptr);
 
     register_mib("smux", (struct variable *) smux_variables,
@@ -1213,7 +1227,7 @@ smux_replace_active(smux_reg * actptr, smux_reg * pasptr)
 }
 
 static void
-smux_list_detach(smux_reg ** head, smux_reg ** m_remove)
+smux_list_detach(smux_reg ** head, smux_reg * m_remove)
 {
     smux_reg       *rptr, *rptr2;
 
@@ -1221,15 +1235,13 @@ smux_list_detach(smux_reg ** head, smux_reg ** m_remove)
         DEBUGMSGTL(("smux", "[smux_list_detach] Ouch!"));
         return;
     }
-    if (*head == *m_remove) {
-        *m_remove = *head;
+    if (*head == m_remove) {
         *head = (*head)->sr_next;
         return;
     }
     for (rptr = *head, rptr2 = rptr->sr_next; rptr2;
          rptr2 = rptr2->sr_next, rptr = rptr->sr_next) {
-        if (rptr2 == *m_remove) {
-            *m_remove = rptr2;
+        if (rptr2 == m_remove) {
             rptr->sr_next = rptr2->sr_next;
             return;
         }
@@ -1335,7 +1347,7 @@ smux_find_replacement(oid * name, size_t name_len)
         if (!snmp_oidtree_compare(rptr->sr_name, rptr->sr_name_len,
                                   name, name_len)) {
             if ((difflen = rptr->sr_name_len - name_len)
-                < bestlen) {
+                < bestlen || !bestptr) {
                 bestlen = difflen;
                 bestptr = rptr;
             } else if ((difflen == bestlen) &&
@@ -1353,7 +1365,7 @@ smux_snmp_process(int exact,
                   size_t * return_len, u_char * return_type, int sd)
 {
     u_char          packet[SMUXMAXPKTSIZE], *ptr, result[SMUXMAXPKTSIZE];
-    size_t          length = SMUXMAXPKTSIZE;
+    ssize_t         length = SMUXMAXPKTSIZE;
     int             tmp_length;
     u_char          type;
     size_t          packet_len;
@@ -1369,7 +1381,7 @@ smux_snmp_process(int exact,
         type = SMUX_GETNEXT;
 
     if (smux_build(type, smux_reqid, objid, len, 0, NULL,
-                   *len, packet, &length) < 0) {
+                   *len, packet, (size_t *) &length) < 0) {
         snmp_log(LOG_ERR, "[smux_snmp_process]: smux_build failed\n");
         return NULL;
     }
@@ -1446,8 +1458,8 @@ smux_snmp_process(int exact,
         if (result[0] == SMUX_TRAP) {
             DEBUGMSGTL(("smux", "[smux_snmp_process] Received trap\n"));
             snmp_log(LOG_INFO, "Got trap from peer on fd %d\n", sd);
-            ptr = asn_parse_header(result, &length, &type);
-            smux_trap_process(ptr, &length);
+            ptr = asn_parse_header(result, (size_t *) &length, &type);
+            smux_trap_process(ptr, (size_t *) &length);
 
             /*
              * go and peek at received data again 
@@ -1744,9 +1756,7 @@ static void
 smux_peer_cleanup(int sd)
 {
     smux_reg       *nrptr, *rptr, *rptr2;
-    int             nfound, i;
-
-    nfound = 0;
+    int             i;
 
     /*
      * close the descriptor 
@@ -1759,7 +1769,7 @@ smux_peer_cleanup(int sd)
     for (rptr = PassiveRegs; rptr; rptr = nrptr) {
         nrptr = rptr->sr_next;
         if (rptr->sr_fd == sd) {
-            smux_list_detach(&PassiveRegs, &rptr);
+            smux_list_detach(&PassiveRegs, rptr);
             free(rptr);
         }
         rptr = nrptr;
@@ -1770,12 +1780,12 @@ smux_peer_cleanup(int sd)
     for (rptr = ActiveRegs; rptr; rptr = rptr2) {
         rptr2 = rptr->sr_next;
         if (rptr->sr_fd == sd) {
-            smux_list_detach(&ActiveRegs, &rptr);
+            smux_list_detach(&ActiveRegs, rptr);
             unregister_mib(rptr->sr_name, rptr->sr_name_len);
             if ((nrptr = smux_find_replacement(rptr->sr_name,
                                                rptr->sr_name_len)) !=
                 NULL) {
-                smux_list_detach(&PassiveRegs, &nrptr);
+                smux_list_detach(&PassiveRegs, nrptr);
                 smux_list_add(&ActiveRegs, nrptr);
                 register_mib("smux", (struct variable *)
                              smux_variables, sizeof(struct variable2),
