@@ -3,6 +3,7 @@
  */
 
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <stdlib.h>
 
 #if HAVE_STRING_H
@@ -17,7 +18,12 @@
 #include "util_funcs/header_generic.h"
 #include "usmUser.h"
 
+#ifndef NETSNMP_NO_WRITE_SUPPORT 
 int usmStatusCheck(struct usmUser *uptr);
+#endif  /* !NETSNMP_NO_WRITE_SUPPORT */
+
+netsnmp_feature_child_of(usmuser_all, libnetsnmpmibs)
+netsnmp_feature_child_of(init_register_usmuser_context, usmuser_all)
 
 struct variable4 usmUser_variables[] = {
     {USMUSERSPINLOCK, ASN_INTEGER, NETSNMP_OLDAPI_RWRITE,
@@ -64,6 +70,7 @@ init_usmUser(void)
                  usmUser_variables_oid);
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_INIT_REGISTER_USMUSER_CONTEXT
 void
 init_register_usmUser_context(const char *contextName) {
     register_mib_context("snmpv3/usmUser",
@@ -75,6 +82,7 @@ init_register_usmUser_context(const char *contextName) {
                          DEFAULT_MIB_PRIORITY, 0, 0, NULL,
                          contextName, -1, 0);
 }
+#endif /* NETSNMP_FEATURE_REMOVE_INIT_REGISTER_USMUSER_CONTEXT */
 
 /*******************************************************************-o-******
  * usm_generate_OID
@@ -288,7 +296,7 @@ var_usmUser(struct variable * vp,
             size_t * length,
             int exact, size_t * var_len, WriteMethod ** write_method)
 {
-    struct usmUser *uptr = NULL, *nptr, *pptr;
+    struct usmUser *uptr = NULL, *nptr;
     int             i, rtest, result;
     oid            *indexOid;
     size_t          len;
@@ -303,8 +311,11 @@ var_usmUser(struct variable * vp,
     if (!vp || !name || !length || !var_len)
         return NULL;
 
-    *write_method = (WriteMethod*)0;    /* assume it isnt writable for the time being */
-    *var_len = sizeof(long_ret);        /* assume an integer and change later if not */
+    /* assume it isnt writable for the time being */
+    *write_method = (WriteMethod*)0;    
+
+    /* assume an integer and change later if not */
+    *var_len = sizeof(long_ret);
 
     if (vp->magic != USMUSERSPINLOCK) {
         oid             newname[MAX_OID_LEN];
@@ -330,8 +341,8 @@ var_usmUser(struct variable * vp,
             uptr = usm_get_userList();
 
         } else {
-            for (nptr = usm_get_userList(), pptr = NULL, uptr = NULL;
-                 nptr != NULL; pptr = nptr, nptr = nptr->next) {
+            for (nptr = usm_get_userList(), uptr = NULL;
+                 nptr != NULL; nptr = nptr->next) {
                 indexOid =
                     usm_generate_OID(vp->name, vp->namelen, nptr, &len);
                 result = snmp_oid_compare(name, *length, indexOid, len);
@@ -374,6 +385,7 @@ var_usmUser(struct variable * vp,
             indexOid = usm_generate_OID(vp->name, vp->namelen, uptr, &len);
             *length = len;
             memmove(name, indexOid, len * sizeof(oid));
+
             DEBUGMSGTL(("usmUser", "Found user: %s - ", uptr->name));
             for (i = 0; i < (int) uptr->engineIDLen; i++) {
                 DEBUGMSG(("usmUser", " %x", uptr->engineID[i]));
@@ -390,6 +402,7 @@ var_usmUser(struct variable * vp,
     }                           /* endif -- vp->magic != USMUSERSPINLOCK */
 
     switch (vp->magic) {
+#ifndef NETSNMP_NO_WRITE_SUPPORT 
     case USMUSERSPINLOCK:
         *write_method = write_usmUserSpinLock;
         long_ret = usmUserSpinLock;
@@ -485,15 +498,21 @@ var_usmUser(struct variable * vp,
             return (unsigned char *) &long_ret;
         }
         return NULL;
-
     default:
         DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_usmUser\n",
                     vp->magic));
+#else /* !NETSNMP_NO_WRITE_SUPPORT */ 
+    default:
+        DEBUGMSGTL(("snmpd", "no write support for var_usmUser\n"));
+#endif /* !NETSNMP_NO_WRITE_SUPPORT */ 
     }
     return NULL;
 
 }                               /* end var_usmUser() */
 
+
+#ifndef NETSNMP_NO_WRITE_SUPPORT 
+
 /*
  * write_usmUserSpinLock(): called when a set is performed on the
  * usmUserSpinLock object 
@@ -747,7 +766,6 @@ write_usmUserAuthProtocol(int action,
         }
     } else if (action == COMMIT) {
         SNMP_FREE(optr);
-        optr = NULL;
     } else if (action == FREE || action == UNDO) {
         if ((uptr = usm_parse_user(name, name_len)) != NULL) {
             if (resetOnFail) {
@@ -890,7 +908,6 @@ write_usmUserAuthKeyChange(int action,
         uptr->authKeyLen = buflen;
     } else if (action == COMMIT) {
         SNMP_FREE(oldkey);
-        oldkey = NULL;
     } else if (action == UNDO) {
         if ((uptr = usm_parse_user(name, name_len)) != NULL && resetOnFail) {
             SNMP_FREE(uptr->authKey);
@@ -1015,7 +1032,6 @@ write_usmUserPrivProtocol(int action,
         }
     } else if (action == COMMIT) {
         SNMP_FREE(optr);
-        optr = NULL;
     } else if (action == FREE || action == UNDO) {
         if ((uptr = usm_parse_user(name, name_len)) != NULL) {
             if (resetOnFail) {
@@ -1140,7 +1156,6 @@ write_usmUserPrivKeyChange(int action,
         uptr->privKeyLen = buflen;
     } else if (action == COMMIT) {
         SNMP_FREE(oldkey);
-        oldkey = NULL;
     } else if (action == UNDO) {
         if ((uptr = usm_parse_user(name, name_len)) != NULL && resetOnFail) {
             SNMP_FREE(uptr->privKey);
@@ -1276,6 +1291,7 @@ write_usmUserStorageType(int action,
     return SNMP_ERR_NOERROR;
 }                               /* end write_usmUserStorageType() */
 
+
 /*
  * Return 1 if enough objects have been set up to transition rowStatus to
  * notInService(2) or active(1).  
@@ -1482,7 +1498,9 @@ write_usmUserStatus(int action,
     }
 
     return SNMP_ERR_NOERROR;
-}
+}  /* write_usmUserStatus */
+
+#endif /* !NETSNMP_NO_WRITE_SUPPORT */ 
 
 #if 0
 
