@@ -1,4 +1,5 @@
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 
 #if HAVE_STRING_H
 #include <string.h>
@@ -25,6 +26,10 @@
 #include "snmpTargetParamsEntry.h"
 #include "target.h"
 
+netsnmp_feature_require(tdomain_support)
+netsnmp_feature_require(tdomain_transport_oid)
+netsnmp_feature_want(netsnmp_tlstmAddr_get_serverId)
+
 #define MAX_TAGS 128
 
 netsnmp_session *
@@ -191,7 +196,7 @@ get_target_sessions(char *taglist, TargetFilterFunction * filterfunct,
 #if defined(NETSNMP_TRANSPORT_DTLSUDP_DOMAIN) || defined(NETSNMP_TRANSPORT_TLSTCP_DOMAIN)
                             if (!tls) {
                                 netsnmp_cert *cert;
-                                char         *server_id;
+                                char         *server_id = NULL;
 
                                 DEBUGMSGTL(("target_sessions",
                                             "  looking up our id: %s\n",
@@ -205,7 +210,7 @@ get_target_sessions(char *taglist, TargetFilterFunction * filterfunct,
                                     DEBUGMSGTL(("target_sessions",
                                             "  found fingerprint: %s\n", 
                                                 cert->fingerprint));
-                                    t->f_config(t, "our_identity",
+                                    t->f_config(t, "localCert",
                                                 cert->fingerprint);
                                 }
                                 DEBUGMSGTL(("target_sessions",
@@ -219,11 +224,13 @@ get_target_sessions(char *taglist, TargetFilterFunction * filterfunct,
                                     DEBUGMSGTL(("target_sessions",
                                             "  found fingerprint: %s\n", 
                                                 cert->fingerprint));
-                                    t->f_config(t, "their_identity",
+                                    t->f_config(t, "peerCert",
                                                 cert->fingerprint);
                                 }
+#ifndef NETSNMP_FEATURE_REMOVE_TLSTMADDR_GET_SERVERID
                                 server_id = netsnmp_tlstmAddr_get_serverId(
                                     targaddrs->name);
+#endif /* NETSNMP_FEATURE_REMOVE_TLSTMADDR_GET_SERVERID */
                                 if (server_id) {
                                     DEBUGMSGTL(("target_sessions",
                                             "  found serverId: %s\n", 
