@@ -1,7 +1,7 @@
 /*
  *  tcpConnTable MIB architecture support
  *
- * $Id: tcpConn_linux.c 18994 2010-06-16 13:13:25Z dts12 $
+ * $Id$
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -56,6 +56,7 @@ netsnmp_arch_tcpconn_entry_copy(netsnmp_tcpconn_entry *lhs,
     return 0;
 }
 
+#ifdef TCPCONN_DELETE_SUPPORTED
 /*
  * delete an entry
  */
@@ -67,7 +68,7 @@ netsnmp_arch_tcpconn_entry_delete(netsnmp_tcpconn_entry *entry)
     /** xxx-rks:9 tcpConn delete not implemented */
     return -1;
 }
-
+#endif /* TCPCONN_DELETE_SUPPORTED */
 
 /**
  *
@@ -83,6 +84,9 @@ netsnmp_arch_tcpconn_container_load(netsnmp_container *container,
     DEBUGMSGTL(("access:tcpconn:container",
                 "tcpconn_container_arch_load (flags %x)\n", load_flags));
 
+    /* Setup the pid_from_inode table, and fill it.*/
+    netsnmp_get_pid_from_inode_init();
+
     if (NULL == container) {
         snmp_log(LOG_ERR, "no container specified/found for access_tcpconn\n");
         return -1;
@@ -134,7 +138,6 @@ _load4(netsnmp_container *container, u_int load_flags)
      */
     while (fgets(line, sizeof(line), in)) {
         netsnmp_tcpconn_entry *entry;
-        int             rc;
         unsigned int    state, local_port, remote_port, tmp_state;
         unsigned long long inode;
         size_t          buf_len, offset;
@@ -183,7 +186,7 @@ _load4(netsnmp_container *container, u_int load_flags)
         entry->rmt_port = (unsigned short) remote_port;
         entry->tcpConnState = state;
         entry->pid = netsnmp_get_pid_from_inode(inode);
-        
+
         /** the addr string may need work */
         buf_len = strlen(local_addr);
         if ((8 != buf_len) ||
@@ -257,27 +260,16 @@ _load6(netsnmp_container *container, u_int load_flags)
     int             rc = 0;
     FILE           *in;
     char            line[180];
-    static int      log_open_err = 1;
 
     netsnmp_assert(NULL != container);
 
 #undef PROCFILE
 #define PROCFILE "/proc/net/tcp6"
     if (!(in = fopen(PROCFILE, "r"))) {
-        snmp_log(LOG_ERR,"could not open " PROCFILE "\n");
-        if (1 == log_open_err) {
-            snmp_log(LOG_ERR,"could not open " PROCFILE "\n");
-            log_open_err = 0;
-        }
+        DEBUGMSGTL(("access:tcpconn:container","could not open " PROCFILE "\n"));
         return -2;
     }
-    /*
-     * if we turned off logging of open errors, turn it back on now that
-     * we have been able to open the file.
-     */
-    if (0 == log_open_err)
-        log_open_err = 1;
-    
+
     fgets(line, sizeof(line), in); /* skip header */
 
     /*
@@ -288,7 +280,7 @@ _load6(netsnmp_container *container, u_int load_flags)
      */
     while (fgets(line, sizeof(line), in)) {
         netsnmp_tcpconn_entry *entry;
-        int             state, rc, local_port, remote_port, tmp_state;
+        int             state, local_port, remote_port, tmp_state;
         unsigned long long  inode;
         size_t          buf_len, offset;
         char            local_addr[48], remote_addr[48];
