@@ -1,7 +1,7 @@
 /*
  * udp_endpoint data access header
  *
- * $Id: udp_endpoint.h 17534 2009-04-23 06:41:55Z magfr $
+ * $Id$
  */
 /**---------------------------------------------------------------------*/
 /*
@@ -23,6 +23,12 @@ config_require(udp-mib/data_access/udp_endpoint_linux)
 config_require(util_funcs/get_pid_from_inode)
 #elif defined( solaris2 )
 config_require(udp-mib/data_access/udp_endpoint_solaris2)
+#elif defined(freebsd4) || defined(dragonfly)
+config_require(udp-mib/data_access/udp_endpoint_freebsd4)
+#elif defined(openbsd4)
+config_require(udp-mib/data_access/udp_endpoint_openbsd)
+#elif defined(netbsd1)
+config_require(udp-mib/data_access/udp_endpoint_netbsd)
 #else
 #   define NETSNMP_UDP_ENDPOINT_COMMON_ONLY
 #endif
