@@ -4,15 +4,14 @@
 #ifndef _MIBGROUP_UTIL_FUNCS_H
 #define _MIBGROUP_UTIL_FUNCS_H
 
-config_require(util_funcs/Exit)
 config_require(util_funcs/header_generic)
 config_require(util_funcs/header_simple_table)
-config_require(util_funcs/restart)
 
-#include "util_funcs/Exit.h"
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
 #include "util_funcs/header_generic.h"
 #include "util_funcs/header_simple_table.h"
-#include "util_funcs/restart.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -37,10 +36,14 @@ int             shell_command(struct extensible *);
 int             exec_command(struct extensible *);
 struct extensible *get_exten_instance(struct extensible *, size_t);
 int             get_exec_output(struct extensible *);
-int             get_exec_pipes(char *cmd, int *fdIn, int *fdOut, int *pid);
+#if defined(WIN32) && !defined(cygwin)
+int             get_exec_pipes(char *cmd, int *fdIn, int *fdOut, HANDLE *pid);
+#else
+int             get_exec_pipes(char *cmd, int *fdIn, int *fdOut, pid_t *pid);
+#endif
 WriteMethod     clear_cache;
 void            print_mib_oid(oid *, size_t);
-void            sprint_mib_oid(char *, oid *, size_t);
+void            sprint_mib_oid(char *, const oid *, size_t);
 int             checkmib(struct variable *, oid *, size_t *, int, size_t *,
                          WriteMethod ** write_method, int);
 char           *find_field(char *, int);
