@@ -7,7 +7,7 @@
 #     modify it under the same terms as Perl itself.
 
 package SNMP;
-$VERSION = '5.0601';   # current release version number
+$VERSION = '5.0702';   # current release version number
 
 use strict;
 use warnings;
@@ -59,7 +59,7 @@ sub AUTOLOAD {
     my $constname;
     ($constname = $AUTOLOAD) =~ s/.*:://;
     # croak "&$module::constant not defined" if $constname eq 'constant';
-    $val = constant($constname, @_ ? $_[0] : 0);
+    ($!, $val) = constant($constname, @_ ? $_[0] : 0);
     if ($! != 0) {
 	if ($! =~ /Invalid/) {
 	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
@@ -135,6 +135,26 @@ $replace_newer = 0; # determine whether or not to tell the parser to replace
                     # older MIB modules with newer ones when loading MIBs.
                     # WARNING: This can cause an incorrect hierarchy.
 
+sub register_debug_tokens {
+    my $tokens = shift;
+
+    SNMP::_register_debug_tokens($tokens);
+}
+
+sub getenv {
+    my $name = shift;
+
+    return SNMP::_getenv($name);
+}
+
+sub setenv {
+    my $envname = shift;
+    my $envval = shift;
+    my $overwrite = shift;
+
+    return SNMP::_setenv($envname, $envval, $overwrite);
+}
+
 sub setMib {
 # loads mib from file name provided
 # setting second arg to true causes currently loaded mib to be replaced
@@ -147,10 +167,10 @@ sub setMib {
 }
 
 sub initMib {
-# eqivalent to calling the snmp library init_mib if Mib is NULL
+# equivalent to calling the snmp library init_mib if Mib is NULL
 # if Mib is already loaded this function does nothing
-# Pass a zero valued argument to get minimal mib tree initialzation
-# If non zero agrgument or no argument then full mib initialization
+# Pass a zero valued argument to get minimal mib tree initialization
+# If non zero argument or no argument then full mib initialization
 
   SNMP::init_snmp("perl");
   return;
@@ -247,8 +267,8 @@ sub mapEnum {
   my $var = shift;
   my ($tag, $val, $update);
   if (ref($var) =~ /ARRAY/ or ref($var) =~ /Varbind/) {
-      $tag = $var->[$SNMP::Varbind::tag_f];
-      $val = $var->[$SNMP::Varbind::val_f];
+      $tag = SNMP::Varbind::tag($var);
+      $val = SNMP::Varbind::val($var);
       $update = 1;
   } else {
       $tag = $var;
@@ -256,7 +276,7 @@ sub mapEnum {
   }
   my $iflag = $val =~ /^\d+$/;
   my $res = SNMP::_map_enum($tag, $val, $iflag, $SNMP::best_guess);
-  if ($update and defined $res) { $var->[$SNMP::Varbind::val_f] = $res; }
+  if ($update and defined $res) { SNMP::Varbind::val($var) = $res; }
   return($res);
 }
 
@@ -763,7 +783,7 @@ sub gettable {
 		# least one column.  We pick the last.
 		push @{$state->{'columns'}}, $root_oid . ".1." .
 		  $children->[$#$children]{'subID'}
-		  if ref($state) eq 'HASH' and ref($children) eq 'HASH';
+		  if ref($state) eq 'HASH' and ref($children) eq 'ARRAY';
 	    }
 	}
     } else {
@@ -807,6 +827,10 @@ sub gettable {
 	# varbinds.
 	$state->{'repeatcount'} = int(1000 / 36 / ($#{$state->{'varbinds'}} + 1));
     }
+    # Make sure we run at least once
+    if ($state->{'repeatcount'} < 1) {
+	$state->{'repeatcount'} = 1;
+    }
 
     #
     # if we've been configured with a callback, then call the
@@ -996,10 +1020,10 @@ sub fget {
    SNMP::_get($this, $this->{RetryNoSuch}, $varbind_list_ref, $cb);
 
    foreach my $varbind (@$varbind_list_ref) {
-     my $sub = $this->{VarFormats}{$varbind->[$SNMP::Varbind::tag_f]} ||
-	 $this->{TypeFormats}{$varbind->[$SNMP::Varbind::type_f]};
+     my $sub = $this->{VarFormats}{SNMP::Varbind::tag($varbind)} ||
+	      $this->{TypeFormats}{SNMP::Varbind::type($varbind)};
      &$sub($varbind) if defined $sub;
-     push(@res, $varbind->[$SNMP::Varbind::val_f]);
+     push(@res, SNMP::Varbind::val($varbind));
    }
 
    return(wantarray() ? @res : $res[0]);
@@ -1049,10 +1073,10 @@ sub fgetnext {
    SNMP::_getnext($this, $varbind_list_ref, $cb);
 
    foreach my $varbind (@$varbind_list_ref) {
-     my $sub = $this->{VarFormats}{$varbind->[$SNMP::Varbind::tag_f]} ||
-	 $this->{TypeFormats}{$varbind->[$SNMP::Varbind::type_f]};
+     my $sub = $this->{VarFormats}{SNMP::Varbind::tag($varbind)} ||
+	      $this->{TypeFormats}{SNMP::Varbind::type($varbind)};
      &$sub($varbind) if defined $sub;
-     push(@res, $varbind->[$SNMP::Varbind::val_f]);
+     push(@res, SNMP::Varbind::val($varbind));
    }
 
    return(wantarray() ? @res : $res[0]);
@@ -1235,11 +1259,11 @@ sub new {
 
 package SNMP::Varbind;
 
-my $tag_f = 0;
-my $iid_f = 1;
-my $val_f = 2;
-my $type_f = 3;
-my $time_f = 4;
+$SNMP::Varbind::tag_f = 0;
+$SNMP::Varbind::iid_f = 1;
+$SNMP::Varbind::val_f = 2;
+$SNMP::Varbind::type_f = 3;
+$SNMP::Varbind::time_f = 4;
 
 sub new {
    my $type = shift;
@@ -1249,27 +1273,27 @@ sub new {
 }
 
 sub tag {
-  $_[0]->[$tag_f];
+  $_[0]->[$SNMP::Varbind::tag_f];
 }
 
 sub iid {
-  $_[0]->[$iid_f];
+  $_[0]->[$SNMP::Varbind::iid_f];
 }
 
 sub val {
-  $_[0]->[$val_f];
+  $_[0]->[$SNMP::Varbind::val_f];
 }
 
 sub type {
-  $_[0]->[$type_f];
+  $_[0]->[$SNMP::Varbind::type_f];
 }
 
 sub name {
-   if (defined($_[0]->[$iid_f]) && ($_[0]->[$iid_f] =~ m/^[0-9]+$/)) {
-      return $_[0]->[$tag_f] . "." . $_[0]->[$iid_f];
+   if (defined($_[0]->[$SNMP::Varbind::iid_f]) && ($_[0]->[$SNMP::Varbind::iid_f] =~ m/^[0-9]+$/)) {
+      return $_[0]->[$SNMP::Varbind::tag_f] . "." . $_[0]->[$SNMP::Varbind::iid_f];
    }
 
-   return $_[0]->[$tag_f];
+   return $_[0]->[$SNMP::Varbind::tag_f];
 }
 
 sub fmt {
@@ -1515,7 +1539,7 @@ init_snmp properly, which means it will read configuration files and
 use those defaults where appropriate automatically parse MIB files,
 etc.  This will likely affect your perl applications if you have, for
 instance, default values set up in your snmp.conf file (as the perl
-module will now make use of those defaults).  The docmuentation,
+module will now make use of those defaults).  The documentation,
 however, has sadly not been updated yet (aside from this note), nor is
 the read_config default usage implementation fully complete.
 
@@ -1526,7 +1550,7 @@ aspects of a connection between the management application and the
 managed agent. Internally the class is implemented as a blessed hash
 reference. This class supplies 'get', 'getnext', 'set', 'fget', and
 'fgetnext' method calls. The methods take a variety of input argument
-formats and support both syncronous and asyncronous operation through
+formats and support both synchronous and asynchronous operation through
 a polymorphic API (i.e., method behaviour varies dependent on args
 passed - see below).
 
@@ -1821,7 +1845,7 @@ do SNMP GET, multiple <vars> formats accepted.
 for syncronous operation <vars> will be updated
 with value(s) and type(s) and will also return
 retrieved value(s). If <callback> supplied method
-will operate asyncronously
+will operate asynchronously
 
 =item $sess->fget(E<lt>varsE<gt> [,E<lt>callbackE<gt>])
 
@@ -1838,7 +1862,7 @@ and <type>
 
 Note: simple string <vars>,(e.g., 'sysDescr.0')
 form is not updated. If <callback> supplied method
-will operate asyncronously
+will operate asynchronously
 
 =item $sess->fgetnext(E<lt>varsE<gt> [,E<lt>callbackE<gt>])
 
@@ -1854,7 +1878,7 @@ format (i.e., well known format) to ensure unambiguous
 translation to SNMP MIB data value (see discussion of
 canonical value format <vars> description section),
 returns snmp_errno. If <callback> supplied method
-will operate asyncronously
+will operate asynchronously
 
 =item $sess->getbulk(E<lt>non-repeatersE<gt>, E<lt>max-repeatersE<gt>, E<lt>varsE<gt>)
 
@@ -1971,7 +1995,7 @@ collect all the columns defined in the MIB table.
 Specifies a GETBULK repeat I<COUNT>.  IE, it will request this many
 varbinds back per column when using the GETBULK operation.  Shortening
 this will mean smaller packets which may help going through some
-systems.  By default, this value is calculated and attepmts to guess
+systems.  By default, this value is calculated and attempts to guess
 at what will fit all the results into 1000 bytes.  This calculation is
 fairly safe, hopefully, but you can either raise or lower the number
 using this option if desired.  In lossy networks, you want to make
@@ -1983,7 +2007,7 @@ one way to help that.
 Force the use of GETNEXT rather than GETBULK.  (always true for
 SNMPv1, as it doesn't have GETBULK anyway).  Some agents are great
 implementers of GETBULK and this allows you to force the use of
-GETNEXT oprations instead.
+GETNEXT operations instead.
 
 =item callback => \&subroutine
 
@@ -2009,7 +2033,7 @@ versions prior to 5.04 and 5.04 and up, the following should work:
       $no_mainloop = 1;
   }
 
-Deciding on whether to use SNMP::MainLoop is left as an excersize to
+Deciding on whether to use SNMP::MainLoop is left as an exercise to
 the reader since it depends on whether your code uses other callbacks
 as well.
 
@@ -2224,9 +2248,9 @@ will be undef.
 to be used with async SNMP::Session
 calls. MainLoop must be called after initial async calls
 so return packets from the agent will not be processed.
-If no args suplied this function enters an infinite loop
+If no args supplied this function enters an infinite loop
 so program must be exited in a callback or externally
-interupted. If <timeout(sic)
+interrupted. If <timeout(sic)
 
 =item &SNMP::finish()
 
@@ -2308,7 +2332,7 @@ initialization
 
 =item $SNMP::debugging
 
-default '0', controlls debugging output level
+default '0', controls debugging output level
 within SNMP module and libsnmp
 
 =over
@@ -2332,6 +2356,12 @@ level 2 plus snmp_set_dump_packet(1)
 default '0', set [non-]zero to independently set
 snmp_set_dump_packet()
 
+=item SNMP::register_debug_tokens()
+
+Allows to register one or more debug tokens, just like the -D option of snmpd.
+Each debug token enables a group of debug statements. An example:
+SNMP::register_debug_tokens("tdomain,netsnmp_unix");
+
 =back
 
 =head1 %SNMP::MIB
@@ -2445,7 +2475,7 @@ returns true if the last object in the INDEX is IMPLIED
 =item &SNMP::setMib(<file>)
 
 allows dynamic parsing of the mib and explicit
-specification of mib file independent of enviroment
+specification of mib file independent of environment
 variables. called with no args acts like initMib,
 loading MIBs indicated by environment variables (see
 Net-SNMP mib_api docs). passing non-zero second arg
