@@ -40,7 +40,7 @@ def _parse_session_args(kargs):
         if sessArgs.has_key(key):
             sessArgs[key] = kargs[key]
         else:
-            print stderr, "ERROR: unknown key", key
+            print >>stderr, "ERROR: unknown key", key
     return sessArgs
 
 def STR(obj):
@@ -57,7 +57,7 @@ class Varbind(object):
         self.type = STR(type)
         # parse iid out of tag if needed
         if iid == None and tag != None:
-            regex = re.compile(r'^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$')
+            regex = re.compile(r'^((?:\.\d+)+|(?:\w+(?:[-:]*\w+)+))\.?(.*)$')
             match = regex.match(tag)
             if match:
                 (self.tag, self.iid) = match.group(1,2)
