@@ -1,6 +1,6 @@
 /*
  * container_binary_array.c
- * $Id: container_binary_array.c 19591 2010-10-29 12:18:51Z bvassche $
+ * $Id$
  *
  * see comments in header file.
  *
@@ -37,7 +37,6 @@ typedef struct binary_array_table_s {
     size_t                     max_size;   /* Size of the current data table */
     size_t                     count;      /* Index of the next free entry */
     int                        dirty;
-    int                        data_size;  /* Size of an individual entry */
     void                     **data;       /* The table itself */
 } binary_array_table;
 
@@ -65,9 +64,9 @@ array_qsort(void **data, int first, int last, netsnmp_container_compare *f)
     mid = data[(first+last)/2];
     
     do {
-        while ( ((*f)(data[i], mid) < 0) && (i < last))
+        while (i < last && (*f)(data[i], mid) < 0)
             ++i;
-        while ( ((*f)(mid, data[j]) < 0) && (j > first))
+        while (j > first && (*f)(mid, data[j]) < 0)
             --j;
 
         if(i < j) {
@@ -221,7 +220,6 @@ netsnmp_binary_array_initialize(void)
     t->max_size = 0;
     t->count = 0;
     t->dirty = 0;
-    t->data_size = sizeof(void*);
     t->data = NULL;
 
     return t;
@@ -231,9 +229,7 @@ void
 netsnmp_binary_array_release(netsnmp_container *c)
 {
     binary_array_table *t = (binary_array_table*)c->container_data;
-    if (t->data != NULL) {
-	SNMP_FREE(t->data);
-    }
+    SNMP_FREE(t->data);
     SNMP_FREE(t);
     SNMP_FREE(c);
 }
@@ -322,7 +318,7 @@ netsnmp_binary_array_remove_at(netsnmp_container *c, size_t index, void **save)
          * otherwise, shift array down
          */
         memmove(&t->data[index], &t->data[index+1],
-                t->data_size * (t->count - index));
+                sizeof(void*) * (t->count - index));
 
         ++c->sync;
     }
@@ -397,16 +393,13 @@ NETSNMP_STATIC_INLINE int
 netsnmp_binary_array_insert(netsnmp_container *c, const void *entry)
 {
     binary_array_table *t = (binary_array_table*)c->container_data;
-    int             new_max, was_dirty = 0;
-    void           *new_data;   /* Used for * a) extending the data table
-                                 * * b) the next entry to use */
+    int             was_dirty = 0;
     /*
      * check for duplicates
      */
     if (! (c->flags & CONTAINER_KEY_ALLOW_DUPLICATES)) {
         was_dirty = t->dirty;
-        new_data = netsnmp_binary_array_get(c, entry, 1);
-        if (NULL != new_data) {
+        if (NULL != netsnmp_binary_array_get(c, entry, 1)) {
             DEBUGMSGTL(("container","not inserting duplicate key\n"));
             return -1;
         }
@@ -417,21 +410,20 @@ netsnmp_binary_array_insert(netsnmp_container *c, const void *entry)
      */
     if (t->max_size <= t->count) {
         /*
-         * Table is full, so extend it to double the size
+         * Table is full, so extend it to double the size, or use 10 elements
+         * if it is empty.
          */
-        new_max = 2 * t->max_size;
-        if (new_max == 0)
-            new_max = 10;       /* Start with 10 entries */
+        size_t const new_max = t->max_size > 0 ? 2 * t->max_size : 10;
+        void ** const new_data =
+            (void**) realloc(t->data, new_max * sizeof(void*));
 
-        new_data = (void *) calloc(new_max, t->data_size);
         if (new_data == NULL)
             return -1;
 
-        if (t->data) {
-            memcpy(new_data, t->data, t->max_size * t->data_size);
-            SNMP_FREE(t->data);
-        }
-        t->data = (void**)new_data;
+        memset(new_data + t->max_size, 0x0,
+               (new_max - t->max_size) * sizeof(void*));
+
+        t->data = new_data;
         t->max_size = new_max;
     }
 
@@ -495,7 +487,7 @@ binary_search_for_start(netsnmp_index *val, netsnmp_container *c)
 void          **
 netsnmp_binary_array_get_subset(netsnmp_container *c, void *key, int *len)
 {
-    binary_array_table *t = (binary_array_table*)c->container_data;
+    binary_array_table *t;
     void          **subset;
     int             start, end;
     size_t          i;
@@ -531,9 +523,9 @@ netsnmp_binary_array_get_subset(netsnmp_container *c, void *key, int *len)
     }
 
     *len = end - start + 1;
-    subset = (void **)malloc((*len) * t->data_size);
+    subset = (void **)malloc((*len) * sizeof(void*));
     if (subset)
-        memcpy(subset, &t->data[start], t->data_size * (*len));
+        memcpy(subset, &t->data[start], sizeof(void*) * (*len));
 
     return subset;
 }
@@ -656,20 +648,19 @@ _ba_duplicate(netsnmp_container *c, void *ctx, u_int flags)
     dupt->max_size = t->max_size;
     dupt->count = t->count;
     dupt->dirty = t->dirty;
-    dupt->data_size = t->data_size;
 
     /*
      * shallow copy
      */
-    dupt->data = (void**) calloc(dupt->max_size, dupt->data_size);
+    dupt->data = (void**) malloc(dupt->max_size * sizeof(void*));
     if (NULL == dupt->data) {
         snmp_log(LOG_ERR, "no memory for binary array duplicate\n");
         netsnmp_binary_array_release(dup);
         return NULL;
     }
 
-    memcpy(dupt->data, t->data, dupt->max_size * dupt->data_size);
-    
+    memcpy(dupt->data, t->data, dupt->max_size * sizeof(void*));
+
     return dup;
 }
 
