@@ -82,30 +82,24 @@ init_snmp_alarm(void)
 void
 sa_update_entry(struct snmp_alarm *a)
 {
-    if (a->t_last.tv_sec == 0 && a->t_last.tv_usec == 0) {
-        struct timeval  t_now;
+    if (!timerisset(&a->t_lastM)) {
         /*
-         * Never been called yet, call time `t' from now.  
+         * First call of sa_update_entry() for alarm a: set t_lastM and t_nextM.
          */
-        gettimeofday(&t_now, NULL);
-
-        a->t_last.tv_sec = t_now.tv_sec;
-        a->t_last.tv_usec = t_now.tv_usec;
-
-        NETSNMP_TIMERADD(&t_now, &a->t, &a->t_next);
-    } else if (a->t_next.tv_sec == 0 && a->t_next.tv_usec == 0) {
+        netsnmp_get_monotonic_clock(&a->t_lastM);
+        NETSNMP_TIMERADD(&a->t_lastM, &a->t, &a->t_nextM);
+    } else if (!timerisset(&a->t_nextM)) {
         /*
          * We've been called but not reset for the next call.  
          */
         if (a->flags & SA_REPEAT) {
-            if (a->t.tv_sec == 0 && a->t.tv_usec == 0) {
+            if (timerisset(&a->t)) {
+                NETSNMP_TIMERADD(&a->t_lastM, &a->t, &a->t_nextM);
+            } else {
                 DEBUGMSGTL(("snmp_alarm",
                             "update_entry: illegal interval specified\n"));
                 snmp_alarm_unregister(a->clientreg);
-                return;
             }
-
-            NETSNMP_TIMERADD(&a->t_last, &a->t, &a->t_next);
         } else {
             /*
              * Single time call, remove it.  
@@ -144,7 +138,7 @@ snmp_alarm_unregister(unsigned int clientreg)
         DEBUGMSGTL(("snmp_alarm", "unregistered alarm %d\n", 
 		    sa_ptr->clientreg));
         /*
-         * Note:  do not free the clientarg, its the clients responsibility 
+         * Note: do not free the clientarg, it's the client's responsibility 
          */
         free(sa_ptr);
     } else {
@@ -178,31 +172,12 @@ struct snmp_alarm *
 sa_find_next(void)
 {
     struct snmp_alarm *a, *lowest = NULL;
-    struct timeval  t_now;
 
-    gettimeofday(&t_now, NULL);
+    for (a = thealarms; a != NULL; a = a->next)
+        if (!(a->flags & SA_FIRED)
+            && (lowest == NULL || timercmp(&a->t_nextM, &lowest->t_nextM, <)))
+            lowest = a;
 
-    for (a = thealarms; a != NULL; a = a->next) {
-        if (!(a->flags & SA_FIRED)) {
-            /* check for time delta skew */
-            if ((a->t_next.tv_sec - t_now.tv_sec) > a->t.tv_sec)
-            {
-                DEBUGMSGTL(("time_skew", "Time delta too big (%ld seconds), should be %ld seconds - fixing\n",
-		    (long)(a->t_next.tv_sec - t_now.tv_sec), (long)a->t.tv_sec));
-                a->t_next.tv_sec = t_now.tv_sec + a->t.tv_sec;
-                a->t_next.tv_usec = t_now.tv_usec + a->t.tv_usec;
-           }
-            if (lowest == NULL) {
-               lowest = a;
-            } else if (a->t_next.tv_sec == lowest->t_next.tv_sec) {
-                if (a->t_next.tv_usec < lowest->t_next.tv_usec) {
-                    lowest = a;
-                }
-            } else if (a->t_next.tv_sec < lowest->t_next.tv_sec) {
-               lowest = a;
-           }
-       }
-    }
     return lowest;
 }
 
@@ -222,8 +197,7 @@ sa_find_specific(unsigned int clientreg)
 void
 run_alarms(void)
 {
-    int             done = 0;
-    struct snmp_alarm *a = NULL;
+    struct snmp_alarm *a;
     unsigned int    clientreg;
     struct timeval  t_now;
 
@@ -232,33 +206,27 @@ run_alarms(void)
      * call until all events are finally in the future again.  
      */
 
-    while (!done) {
-        if ((a = sa_find_next()) == NULL) {
+    while ((a = sa_find_next()) != NULL) {
+        netsnmp_get_monotonic_clock(&t_now);
+
+        if (timercmp(&a->t_nextM, &t_now, >))
             return;
-        }
 
-        gettimeofday(&t_now, NULL);
+        clientreg = a->clientreg;
+        a->flags |= SA_FIRED;
+        DEBUGMSGTL(("snmp_alarm", "run alarm %d\n", clientreg));
+        (*(a->thecallback)) (clientreg, a->clientarg);
+        DEBUGMSGTL(("snmp_alarm", "alarm %d completed\n", clientreg));
 
-        if (timercmp(&a->t_next, &t_now, <)) {
-            clientreg = a->clientreg;
-            a->flags |= SA_FIRED;
-            DEBUGMSGTL(("snmp_alarm", "run alarm %d\n", clientreg));
-            (*(a->thecallback)) (clientreg, a->clientarg);
-            DEBUGMSGTL(("snmp_alarm", "alarm %d completed\n", clientreg));
-
-            if ((a = sa_find_specific(clientreg)) != NULL) {
-                a->t_last.tv_sec = t_now.tv_sec;
-                a->t_last.tv_usec = t_now.tv_usec;
-                a->t_next.tv_sec = 0;
-                a->t_next.tv_usec = 0;
-                a->flags &= ~SA_FIRED;
-                sa_update_entry(a);
-            } else {
-                DEBUGMSGTL(("snmp_alarm", "alarm %d deleted itself\n",
-                            clientreg));
-            }
+        a = sa_find_specific(clientreg);
+        if (a) {
+            a->t_lastM = t_now;
+            timerclear(&a->t_nextM);
+            a->flags &= ~SA_FIRED;
+            sa_update_entry(a);
         } else {
-            done = 1;
+            DEBUGMSGTL(("snmp_alarm", "alarm %d deleted itself\n",
+                        clientreg));
         }
     }
 }
@@ -274,39 +242,56 @@ alarm_handler(int a)
 
 
 
+/**
+ * Look up the time at which the next alarm will fire.
+ *
+ * @param[out] alarm_tm Time at which the next alarm will fire.
+ * @param[in] now Earliest time that should be written into *alarm_tm.
+ *
+ * @return Zero if no alarms are scheduled; non-zero 'clientreg' value
+ *   identifying the first alarm that will fire if one or more alarms are
+ *   scheduled.
+ */
 int
-get_next_alarm_delay_time(struct timeval *delta)
+netsnmp_get_next_alarm_time(struct timeval *alarm_tm, const struct timeval *now)
 {
     struct snmp_alarm *sa_ptr;
-    struct timeval  t_now;
 
     sa_ptr = sa_find_next();
 
     if (sa_ptr) {
-        gettimeofday(&t_now, NULL);
-
-        if (timercmp(&t_now, &sa_ptr->t_next, >)) {
-            /*
-             * Time has already passed.  Return the smallest possible amount of
-             * time.  
-             */
-            delta->tv_sec = 0;
-            delta->tv_usec = 1;
-            return sa_ptr->clientreg;
-        } else {
-            /*
-             * Time is still in the future.  
-             */
-            NETSNMP_TIMERSUB(&sa_ptr->t_next, &t_now, delta);
-
-            return sa_ptr->clientreg;
-        }
+        netsnmp_assert(alarm_tm);
+        netsnmp_assert(timerisset(&sa_ptr->t_nextM));
+        if (timercmp(&sa_ptr->t_nextM, now, >))
+            *alarm_tm = sa_ptr->t_nextM;
+        else
+            *alarm_tm = *now;
+        return sa_ptr->clientreg;
+    } else {
+        return 0;
     }
+}
 
-    /*
-     * Nothing Left.  
-     */
-    return 0;
+/**
+ * Get the time until the next alarm will fire.
+ *
+ * @param[out] delta Time until the next alarm.
+ *
+ * @return Zero if no alarms are scheduled; non-zero 'clientreg' value
+ *   identifying the first alarm that will fire if one or more alarms are
+ *   scheduled.
+ */
+int
+get_next_alarm_delay_time(struct timeval *delta)
+{
+    struct timeval t_now, alarm_tm;
+    int res;
+
+    netsnmp_get_monotonic_clock(&t_now);
+    res = netsnmp_get_next_alarm_time(&alarm_tm, &t_now);
+    if (res)
+        NETSNMP_TIMERSUB(&alarm_tm, &t_now, delta);
+    return res;
 }
 
 
@@ -328,15 +313,13 @@ set_an_alarm(void)
 # ifdef HAVE_SETITIMER
         struct itimerval it;
 
-        it.it_value.tv_sec = delta.tv_sec;
-        it.it_value.tv_usec = delta.tv_usec;
-        it.it_interval.tv_sec = 0;
-        it.it_interval.tv_usec = 0;
+        it.it_value = delta;
+        timerclear(&it.it_interval);
 
         signal(SIGALRM, alarm_handler);
         setitimer(ITIMER_REAL, &it, NULL);
         DEBUGMSGTL(("snmp_alarm", "schedule alarm %d in %ld.%03ld seconds\n",
-                    nextalarm, delta.tv_sec, (delta.tv_usec / 1000)));
+                    nextalarm, (long) delta.tv_sec, (delta.tv_usec / 1000)));
 # else  /* HAVE_SETITIMER */
 #  ifdef SIGALRM
         signal(SIGALRM, alarm_handler);
@@ -452,8 +435,7 @@ snmp_alarm_register_hr(struct timeval t, unsigned int flags,
         return 0;
     }
 
-    (*s)->t.tv_sec = t.tv_sec;
-    (*s)->t.tv_usec = t.tv_usec;
+    (*s)->t = t;
     (*s)->flags = flags;
     (*s)->clientarg = cd;
     (*s)->thecallback = cb;
@@ -464,7 +446,7 @@ snmp_alarm_register_hr(struct timeval t, unsigned int flags,
 
     DEBUGMSGTL(("snmp_alarm",
                 "registered alarm %d, t = %ld.%03ld, flags=0x%02x\n",
-                (*s)->clientreg, (*s)->t.tv_sec, ((*s)->t.tv_usec / 1000),
+                (*s)->clientreg, (long) (*s)->t.tv_sec, ((*s)->t.tv_usec / 1000),
                 (*s)->flags));
 
     if (start_alarms) {
@@ -473,4 +455,35 @@ snmp_alarm_register_hr(struct timeval t, unsigned int flags,
 
     return (*s)->clientreg;
 }
+
+/**
+ * This function resets an existing alarm.
+ *
+ * @param clientreg is a unique unsigned integer representing a registered
+ *	alarm which the client wants to unregister.
+ *
+ * @return 0 on success, -1 if the alarm was not found
+ *
+ * @see snmp_alarm_register
+ * @see snmp_alarm_register_hr
+ * @see snmp_alarm_unregister
+ */
+int
+snmp_alarm_reset(unsigned int clientreg)
+{
+    struct snmp_alarm *a;
+    struct timeval  t_now;
+    if ((a = sa_find_specific(clientreg)) != NULL) {
+        netsnmp_get_monotonic_clock(&t_now);
+        a->t_lastM.tv_sec = t_now.tv_sec;
+        a->t_lastM.tv_usec = t_now.tv_usec;
+        a->t_nextM.tv_sec = 0;
+        a->t_nextM.tv_usec = 0;
+        NETSNMP_TIMERADD(&t_now, &a->t, &a->t_nextM);
+        return 0;
+    }
+    DEBUGMSGTL(("snmp_alarm_reset", "alarm %d not found\n",
+                clientreg));
+    return -1;
+}
 /**  @} */
