@@ -155,12 +155,12 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
       }
 
     /*
-     * Generate the Command to be executed by SCM 
+     * Generate the command to be executed by the SCM 
      */
     _sntprintf (szServiceCommand, CountOf(szServiceCommand), _T("%s %s"), szServicePath, _T ("-service"));
 
     /*
-     * Create the Desired service 
+     * Create the desired service 
      */
     hService = CreateService (hSCManager, lpszServiceName, lpszServiceDisplayName,
 			SERVICE_ALL_ACCESS, SERVICE_WIN32_OWN_PROCESS,
@@ -181,7 +181,7 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
       }
 
     /*
-     * Create registry entries for EventLog 
+     * Create registry entries for the event log 
      */
     /*
      * Create registry Application event log key 
@@ -329,7 +329,7 @@ RegisterService (LPCTSTR lpszServiceName, LPCTSTR lpszServiceDisplayName,
       }
 
     /*
-     * Ready to Log messages 
+     * Ready to log messages 
      */
 
     /*
@@ -456,8 +456,7 @@ UnregisterService (LPCTSTR lpszServiceName, int quiet)
 }
 
     /*
-     * To write message to Windows Event log
-     * Input - Event Type, Message string
+     * Write a message to the Windows event log.
      */
 VOID
 WriteToEventLog (WORD wType, LPCTSTR pszFormat, ...)
@@ -466,6 +465,7 @@ WriteToEventLog (WORD wType, LPCTSTR pszFormat, ...)
   LPCTSTR LogStr[1];
   va_list ArgList;
   HANDLE hEventSource = NULL;
+
   va_start (ArgList, pszFormat);
   _vsntprintf (szMessage, CountOf(szMessage), pszFormat, ArgList);
   va_end (ArgList);
@@ -474,7 +474,7 @@ WriteToEventLog (WORD wType, LPCTSTR pszFormat, ...)
   if (hEventSource == NULL)
     return;
   ReportEvent (hEventSource, wType, 0,
-	       DISPLAY_MSG,	/* To Just output the text to event log */
+	       DISPLAY_MSG,
 	       NULL, 1, 0, LogStr, NULL);
   DeregisterEventSource (hEventSource);
 }
@@ -492,7 +492,7 @@ WriteToEventLog (WORD wType, LPCTSTR pszFormat, ...)
 INT
 ParseCmdLineForServiceOption (int argc, TCHAR * argv[], int *quiet)
 {
-  int nReturn = RUN_AS_CONSOLE;	/* Defualted to run as console */
+  int nReturn = RUN_AS_CONSOLE;	/* default is to run as a console application */
 
   if (argc >= 2)
     {
@@ -531,7 +531,7 @@ ParseCmdLineForServiceOption (int argc, TCHAR * argv[], int *quiet)
 }
 
     /*
-     * Write error message to Event Log, console or pop-up window
+     * Write error message to event log, console or pop-up window.
      *
      * If useGetLastError is 1, the last error returned from GetLastError()
      * is appended to pszMessage, separated by a ": ".
@@ -572,13 +572,13 @@ ProcessError (WORD eventLogType, LPCTSTR pszMessage, int useGetLastError, int qu
   
   hEventSource = RegisterEventSource (NULL, app_name_long);
   if (hEventSource != NULL) {
-    LPCSTR LogStr[1];
+    LPCTSTR LogStr[1];
     LogStr[0] = pszMessageFull;
     
     if (ReportEvent (hEventSource, 
           eventLogType, 
           0,
-          DISPLAY_MSG,	/* To Just output the text to event log */
+          DISPLAY_MSG,	/* just output the text to the event log */
           NULL, 
           1, 
           0, 
@@ -622,9 +622,9 @@ ProcessError (WORD eventLogType, LPCTSTR pszMessage, int useGetLastError, int qu
 }
 
     /*
-     *  To update current service status 
-     *  Sends the current service status to the SCM. Also updates
-     *  the global service status structure.
+     * Update current service status.
+     * Sends the current service status to the SCM. Also updates
+     * the global service status structure.
      */
 static BOOL
 UpdateServiceStatus (DWORD dwStatus, DWORD dwErrorCode, DWORD dwWaitHint)
@@ -662,7 +662,7 @@ UpdateServiceStatus (DWORD dwStatus, DWORD dwErrorCode, DWORD dwWaitHint)
 }
 
     /*
-     * Reports current Service status to SCM
+     * Reports current service status to SCM
      */
 static BOOL
 ReportCurrentServiceStatus ()
@@ -671,7 +671,7 @@ ReportCurrentServiceStatus ()
 }
 
     /*
-     * The ServiceMain function to start service.
+     * ServiceMain function.
      */
 VOID WINAPI
 ServiceMain (DWORD argc, LPTSTR argv[])
@@ -680,14 +680,12 @@ ServiceMain (DWORD argc, LPTSTR argv[])
   unsigned threadId;
 
   /*
-   * Input Arguments to function startup 
+   * Input arguments
    */
   DWORD ArgCount = 0;
   LPTSTR *ArgArray = NULL;
   TCHAR szRegKey[512];
-  TCHAR szValue[128];
-  DWORD nSize;
-  HKEY hParamKey = NULL;	/* To read startup parameters */
+  HKEY hParamKey = NULL;
   DWORD TotalParams = 0;
   DWORD i;
   InputParams ThreadInputParams;
@@ -698,7 +696,7 @@ ServiceMain (DWORD argc, LPTSTR argv[])
 
   /*
    * SCM sends Service Name as first arg, increment to point
-   * arguments user specified while starting contorl agent
+   * arguments user specified while starting control agent
    */
 
   /*
@@ -707,7 +705,7 @@ ServiceMain (DWORD argc, LPTSTR argv[])
   ArgCount = 1;
 
   /*
-   * Create Registry Key path 
+   * Create registry key path 
    */
   _sntprintf (szRegKey, CountOf(szRegKey), _T("%s%s\\%s"),
 	     _T ("SYSTEM\\CurrentControlSet\\Services\\"), app_name_long,
@@ -717,7 +715,7 @@ ServiceMain (DWORD argc, LPTSTR argv[])
     {
 
       /*
-       * Read startup Configuration information 
+       * Read startup configuration information 
        */
       /*
        * Find number of subkeys inside parameters 
@@ -733,7 +731,7 @@ ServiceMain (DWORD argc, LPTSTR argv[])
 	      /*
 	       * Allocate memory to hold strings 
 	       */
-	      ArgArray = (LPTSTR *) malloc (sizeof (LPTSTR) * ArgCount);
+	      ArgArray = calloc(ArgCount, sizeof(ArgArray[0]));
               if (ArgArray == 0)
                 {
                   WriteToEventLog (EVENTLOG_ERROR_TYPE,
@@ -747,6 +745,10 @@ ServiceMain (DWORD argc, LPTSTR argv[])
 	      ArgArray[0] = _tcsdup (argv[0]);
 	      for (i = 1; i <= TotalParams; i++)
 		{
+                  DWORD dwErrorcode;
+                  DWORD nSize;
+                  DWORD nRegkeyType;
+                  TCHAR *szValue;
 
 		  /*
 		   * Create Subkey value name 
@@ -754,12 +756,35 @@ ServiceMain (DWORD argc, LPTSTR argv[])
 		  _sntprintf (szRegKey, CountOf(szRegKey), _T("%s%d"), _T("Param"), i);
 
 		  /*
-		   * Set size 
+		   * Query subkey.
 		   */
-		  nSize = 128;
-		  RegQueryValueEx (hParamKey, szRegKey, 0, NULL,
-				   (LPBYTE) & szValue, &nSize);
-		  ArgArray[i] = _tcsdup (szValue);
+		  nSize = 0;
+		  dwErrorcode = RegQueryValueEx(hParamKey, szRegKey, NULL,
+                                                &nRegkeyType, NULL, &nSize);
+                  if (dwErrorcode == ERROR_SUCCESS) {
+                    if (nRegkeyType == REG_SZ || nRegkeyType == REG_EXPAND_SZ) {
+                      szValue = malloc(nSize + sizeof(szValue[0]));
+                      if (szValue) {
+		        dwErrorcode = RegQueryValueEx(hParamKey, szRegKey, NULL,
+                                                      &nRegkeyType, (LPBYTE)szValue, &nSize);
+                        if (dwErrorcode == ERROR_SUCCESS) {
+                          szValue[nSize] = 0;
+                          ArgArray[i] = szValue;
+                        } else {
+                          free(szValue);
+                          WriteToEventLog(EVENTLOG_ERROR_TYPE, _T("Querying registry key %s failed: error code %ld"), szRegKey, dwErrorcode);
+                        }
+                      } else
+                        WriteToEventLog(EVENTLOG_ERROR_TYPE, _T("Querying registry key %s failed: out of memory"), szRegKey);
+                    } else
+                      WriteToEventLog(EVENTLOG_ERROR_TYPE, _T("Type %ld of registry key %s is incorrect"), nRegkeyType, szRegKey);
+                  } else
+                    WriteToEventLog(EVENTLOG_ERROR_TYPE, _T("Querying registry key %s failed: error code %ld"), szRegKey, dwErrorcode);
+
+                  if (!ArgArray[i]) {
+                    TotalParams = ArgCount = i;
+                    break;
+                  }
 		}
 	    }
 	}
@@ -769,7 +794,7 @@ ServiceMain (DWORD argc, LPTSTR argv[])
     {
 
       /*
-       * No statup agrs are given 
+       * No startup args are given 
        */
       ThreadInputParams.Argc = argc;
       ThreadInputParams.Argv = argv;
@@ -793,12 +818,12 @@ ServiceMain (DWORD argc, LPTSTR argv[])
     }
 
   /*
-   * Update the service status to START_PENDING 
+   * Update the service status to START_PENDING.
    */
   UpdateServiceStatus (SERVICE_START_PENDING, NO_ERROR, SCM_WAIT_INTERVAL);
 
   /*
-   * Spin of worker thread, which does majority of the work 
+   * Start the worker thread, which does the majority of the work .
    */
   TRY
   {
@@ -819,13 +844,12 @@ ServiceMain (DWORD argc, LPTSTR argv[])
       }
 
     /*
-     * Set Service Status to Running 
+     * Set service status to SERVICE_RUNNING.
      */
     UpdateServiceStatus (SERVICE_RUNNING, NO_ERROR, SCM_WAIT_INTERVAL);
 
     /*
-     * Wait for termination event and worker thread to
-     * * spin down.
+     * Wait until the worker thread finishes.
      */
     WaitForSingleObject (hServiceThread, INFINITE);
   }
@@ -840,12 +864,12 @@ ServiceMain (DWORD argc, LPTSTR argv[])
     FreeSecurityAttributes (&SecurityAttributes);
 
     /*
-     * Delete allocated argument list 
+     * Free allocated argument list 
      */
     if (ArgCount > 1 && ArgArray != NULL)
       {
 	/*
-	 * Delete all strings 
+	 * Free all strings 
 	 */
 	for (i = 0; i < ArgCount; i++)
 	  {
